import json
from typing import List, Optional, Tuple, Union

import grpc

from nibiru.clients import Dex as DexClient
from nibiru.clients import Perp as PerpClient
from nibiru.clients import Pricefeed as PricefeedClient
from nibiru.clients import VPool as VPoolClient
from nibiru.exceptions import NotFoundError
from nibiru.network import Network
from nibiru.proto.cosmos.auth.v1beta1 import auth_pb2 as auth_type
from nibiru.proto.cosmos.auth.v1beta1 import query_pb2 as auth_query
from nibiru.proto.cosmos.auth.v1beta1 import query_pb2_grpc as auth_query_grpc
from nibiru.proto.cosmos.authz.v1beta1 import query_pb2 as authz_query
from nibiru.proto.cosmos.authz.v1beta1 import query_pb2_grpc as authz_query_grpc
from nibiru.proto.cosmos.bank.v1beta1 import query_pb2 as bank_query
from nibiru.proto.cosmos.bank.v1beta1 import query_pb2_grpc as bank_query_grpc
from nibiru.proto.cosmos.base.abci.v1beta1 import abci_pb2 as abci_type
from nibiru.proto.cosmos.base.tendermint.v1beta1 import query_pb2 as tendermint_query
from nibiru.proto.cosmos.base.tendermint.v1beta1 import (
    query_pb2_grpc as tendermint_query_grpc,
)
from nibiru.proto.cosmos.tx.v1beta1 import service_pb2 as tx_service
from nibiru.proto.cosmos.tx.v1beta1 import service_pb2_grpc as tx_service_grpc

DEFAULT_TIMEOUTHEIGHT = 20  # blocks


class Client:
    def __init__(
        self,
        network: Network,
        insecure=False,
        credentials: grpc.ChannelCredentials = None,
    ):

        # load root CA cert
        if not insecure:
            credentials = grpc.ssl_channel_credentials()

        # chain stubs
        self.chain_channel = (
            grpc.insecure_channel(network.grpc_endpoint)
            if insecure
            else grpc.secure_channel(network.grpc_endpoint, credentials)
        )
        self.insecure = insecure
        self.stubCosmosTendermint = tendermint_query_grpc.ServiceStub(self.chain_channel)
        self.stubAuth = auth_query_grpc.QueryStub(self.chain_channel)
        self.stubAuthz = authz_query_grpc.QueryStub(self.chain_channel)
        self.stubBank = bank_query_grpc.QueryStub(self.chain_channel)
        self.stubTx = tx_service_grpc.ServiceStub(self.chain_channel)

        self.timeout_height = 1

        # Query services
        self.dex = DexClient(self.chain_channel)
        self.pricefeed = PricefeedClient(self.chain_channel)
        self.perp = PerpClient(self.chain_channel)
        self.vpool = VPoolClient(self.chain_channel)

    def close_chain_channel(self):
        self.chain_channel.close()

    def sync_timeout_height(self):
        block = self.get_latest_block()
        self.timeout_height = block.block.header.height + DEFAULT_TIMEOUTHEIGHT

    # default client methods
    def get_latest_block(self) -> tendermint_query.GetLatestBlockResponse:
        req = tendermint_query.GetLatestBlockRequest()
        return self.stubCosmosTendermint.GetLatestBlock(req)

    def get_account(self, address: str) -> Optional[auth_type.BaseAccount]:
        try:
            account_any = self.stubAuth.Account(auth_query.QueryAccountRequest(address=address)).account
            account = auth_type.BaseAccount()
            if account_any.Is(account.DESCRIPTOR):
                account_any.Unpack(account)
                return account
        except:
            return None

    def get_request_id_by_tx_hash(self, tx_hash: bytes) -> List[int]:
        tx = self.stubTx.GetTx(tx_service.GetTxRequest(hash=tx_hash))
        request_ids = []
        for tx in tx.tx_response.logs:
            request_event = [event for event in tx.events if event.type == "request" or event.type == "report"]
            if len(request_event) == 1:
                attrs = request_event[0].attributes
                attr_id = [attr for attr in attrs if attr.key == "id"]
                if len(attr_id) == 1:
                    request_id = attr_id[0].value
                    request_ids.append(int(request_id))
        if len(request_ids) == 0:
            raise NotFoundError("Request Id is not found")
        return request_ids

    def simulate_tx(self, tx_byte: bytes) -> Tuple[Union[abci_type.SimulationResponse, grpc.RpcError], bool]:
        try:
            req = tx_service.SimulateRequest(tx_bytes=tx_byte)
            return self.stubTx.Simulate.__call__(req), True
        except grpc.RpcError as err:
            return json.loads(err._state.debug_error_string)["grpc_message"], False

    def send_tx_sync_mode(self, tx_byte: bytes) -> abci_type.TxResponse:
        req = tx_service.BroadcastTxRequest(tx_bytes=tx_byte, mode=tx_service.BroadcastMode.BROADCAST_MODE_SYNC)
        result = self.stubTx.BroadcastTx.__call__(req)
        return result.tx_response

    def send_tx_async_mode(self, tx_byte: bytes) -> abci_type.TxResponse:
        req = tx_service.BroadcastTxRequest(tx_bytes=tx_byte, mode=tx_service.BroadcastMode.BROADCAST_MODE_ASYNC)
        result = self.stubTx.BroadcastTx.__call__(req)
        return result.tx_response

    def send_tx_block_mode(self, tx_byte: bytes) -> abci_type.TxResponse:
        req = tx_service.BroadcastTxRequest(tx_bytes=tx_byte, mode=tx_service.BroadcastMode.BROADCAST_MODE_BLOCK)
        result = self.stubTx.BroadcastTx.__call__(req)
        return result.tx_response

    def get_chain_id(self) -> str:
        latest_block = self.get_latest_block()
        return latest_block.block.header.chain_id

    def get_grants(self, granter: str, grantee: str, **kwargs):
        return self.stubAuthz.Grants(
            authz_query.QueryGrantsRequest(
                granter=granter,
                grantee=grantee,
                msg_type_url=kwargs.get("msg_type_url"),
            )
        )

    def get_bank_balances(self, address: str):
        return self.stubBank.AllBalances(bank_query.QueryAllBalancesRequest(address=address))

    def get_bank_balance(self, address: str, denom: str):
        return self.stubBank.Balance(bank_query.QueryBalanceRequest(address=address, denom=denom))



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import cosmos.crypto.multisig.v1beta1.multisig_pb2
import google.protobuf.any_pb2
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _SignMode:
    ValueType = typing.NewType('ValueType', builtins.int)
    V: typing_extensions.TypeAlias = ValueType
class _SignModeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_SignMode.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    SIGN_MODE_UNSPECIFIED: _SignMode.ValueType  # 0
    """SIGN_MODE_UNSPECIFIED specifies an unknown signing mode and will be
    rejected
    """

    SIGN_MODE_DIRECT: _SignMode.ValueType  # 1
    """SIGN_MODE_DIRECT specifies a signing mode which uses SignDoc and is
    verified with raw bytes from Tx
    """

    SIGN_MODE_TEXTUAL: _SignMode.ValueType  # 2
    """SIGN_MODE_TEXTUAL is a future signing mode that will verify some
    human-readable textual representation on top of the binary representation
    from SIGN_MODE_DIRECT
    """

    SIGN_MODE_LEGACY_AMINO_JSON: _SignMode.ValueType  # 127
    """SIGN_MODE_LEGACY_AMINO_JSON is a backwards compatibility mode which uses
    Amino JSON and will be removed in the future
    """

    SIGN_MODE_EIP_191: _SignMode.ValueType  # 191
    """SIGN_MODE_EIP_191 specifies the sign mode for EIP 191 signing on the Cosmos
    SDK. Ref: https://eips.ethereum.org/EIPS/eip-191

    Currently, SIGN_MODE_EIP_191 is registered as a SignMode enum variant,
    but is not implemented on the SDK by default. To enable EIP-191, you need
    to pass a custom `TxConfig` that has an implementation of
    `SignModeHandler` for EIP-191. The SDK may decide to fully support
    EIP-191 in the future.

    Since: cosmos-sdk 0.45.2
    """

class SignMode(_SignMode, metaclass=_SignModeEnumTypeWrapper):
    """SignMode represents a signing mode with its own security guarantees."""
    pass

SIGN_MODE_UNSPECIFIED: SignMode.ValueType  # 0
"""SIGN_MODE_UNSPECIFIED specifies an unknown signing mode and will be
rejected
"""

SIGN_MODE_DIRECT: SignMode.ValueType  # 1
"""SIGN_MODE_DIRECT specifies a signing mode which uses SignDoc and is
verified with raw bytes from Tx
"""

SIGN_MODE_TEXTUAL: SignMode.ValueType  # 2
"""SIGN_MODE_TEXTUAL is a future signing mode that will verify some
human-readable textual representation on top of the binary representation
from SIGN_MODE_DIRECT
"""

SIGN_MODE_LEGACY_AMINO_JSON: SignMode.ValueType  # 127
"""SIGN_MODE_LEGACY_AMINO_JSON is a backwards compatibility mode which uses
Amino JSON and will be removed in the future
"""

SIGN_MODE_EIP_191: SignMode.ValueType  # 191
"""SIGN_MODE_EIP_191 specifies the sign mode for EIP 191 signing on the Cosmos
SDK. Ref: https://eips.ethereum.org/EIPS/eip-191

Currently, SIGN_MODE_EIP_191 is registered as a SignMode enum variant,
but is not implemented on the SDK by default. To enable EIP-191, you need
to pass a custom `TxConfig` that has an implementation of
`SignModeHandler` for EIP-191. The SDK may decide to fully support
EIP-191 in the future.

Since: cosmos-sdk 0.45.2
"""

global___SignMode = SignMode


class SignatureDescriptors(google.protobuf.message.Message):
    """SignatureDescriptors wraps multiple SignatureDescriptor's."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    SIGNATURES_FIELD_NUMBER: builtins.int
    @property
    def signatures(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___SignatureDescriptor]:
        """signatures are the signature descriptors"""
        pass
    def __init__(self,
        *,
        signatures: typing.Optional[typing.Iterable[global___SignatureDescriptor]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["signatures",b"signatures"]) -> None: ...
global___SignatureDescriptors = SignatureDescriptors

class SignatureDescriptor(google.protobuf.message.Message):
    """SignatureDescriptor is a convenience type which represents the full data for
    a signature including the public key of the signer, signing modes and the
    signature itself. It is primarily used for coordinating signatures between
    clients.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    class Data(google.protobuf.message.Message):
        """Data represents signature data"""
        DESCRIPTOR: google.protobuf.descriptor.Descriptor
        class Single(google.protobuf.message.Message):
            """Single is the signature data for a single signer"""
            DESCRIPTOR: google.protobuf.descriptor.Descriptor
            MODE_FIELD_NUMBER: builtins.int
            SIGNATURE_FIELD_NUMBER: builtins.int
            mode: global___SignMode.ValueType
            """mode is the signing mode of the single signer"""

            signature: builtins.bytes
            """signature is the raw signature bytes"""

            def __init__(self,
                *,
                mode: global___SignMode.ValueType = ...,
                signature: builtins.bytes = ...,
                ) -> None: ...
            def ClearField(self, field_name: typing_extensions.Literal["mode",b"mode","signature",b"signature"]) -> None: ...

        class Multi(google.protobuf.message.Message):
            """Multi is the signature data for a multisig public key"""
            DESCRIPTOR: google.protobuf.descriptor.Descriptor
            BITARRAY_FIELD_NUMBER: builtins.int
            SIGNATURES_FIELD_NUMBER: builtins.int
            @property
            def bitarray(self) -> cosmos.crypto.multisig.v1beta1.multisig_pb2.CompactBitArray:
                """bitarray specifies which keys within the multisig are signing"""
                pass
            @property
            def signatures(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___SignatureDescriptor.Data]:
                """signatures is the signatures of the multi-signature"""
                pass
            def __init__(self,
                *,
                bitarray: typing.Optional[cosmos.crypto.multisig.v1beta1.multisig_pb2.CompactBitArray] = ...,
                signatures: typing.Optional[typing.Iterable[global___SignatureDescriptor.Data]] = ...,
                ) -> None: ...
            def HasField(self, field_name: typing_extensions.Literal["bitarray",b"bitarray"]) -> builtins.bool: ...
            def ClearField(self, field_name: typing_extensions.Literal["bitarray",b"bitarray","signatures",b"signatures"]) -> None: ...

        SINGLE_FIELD_NUMBER: builtins.int
        MULTI_FIELD_NUMBER: builtins.int
        @property
        def single(self) -> global___SignatureDescriptor.Data.Single:
            """single represents a single signer"""
            pass
        @property
        def multi(self) -> global___SignatureDescriptor.Data.Multi:
            """multi represents a multisig signer"""
            pass
        def __init__(self,
            *,
            single: typing.Optional[global___SignatureDescriptor.Data.Single] = ...,
            multi: typing.Optional[global___SignatureDescriptor.Data.Multi] = ...,
            ) -> None: ...
        def HasField(self, field_name: typing_extensions.Literal["multi",b"multi","single",b"single","sum",b"sum"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing_extensions.Literal["multi",b"multi","single",b"single","sum",b"sum"]) -> None: ...
        def WhichOneof(self, oneof_group: typing_extensions.Literal["sum",b"sum"]) -> typing.Optional[typing_extensions.Literal["single","multi"]]: ...

    PUBLIC_KEY_FIELD_NUMBER: builtins.int
    DATA_FIELD_NUMBER: builtins.int
    SEQUENCE_FIELD_NUMBER: builtins.int
    @property
    def public_key(self) -> google.protobuf.any_pb2.Any:
        """public_key is the public key of the signer"""
        pass
    @property
    def data(self) -> global___SignatureDescriptor.Data: ...
    sequence: builtins.int
    """sequence is the sequence of the account, which describes the
    number of committed transactions signed by a given address. It is used to prevent
    replay attacks.
    """

    def __init__(self,
        *,
        public_key: typing.Optional[google.protobuf.any_pb2.Any] = ...,
        data: typing.Optional[global___SignatureDescriptor.Data] = ...,
        sequence: builtins.int = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["data",b"data","public_key",b"public_key"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["data",b"data","public_key",b"public_key","sequence",b"sequence"]) -> None: ...
global___SignatureDescriptor = SignatureDescriptor



import  dataclasses
import os
from typing import Dict, Optional 

@dataclasses.dataclass
class Network:
    lcd_endpoint: str
    grpc_endpoint: str
    chain_id: str
    fee_denom: str
    env: str

    @classmethod
    def devnet(cls) -> "Network":
        chain_config: Dict[str, Optional[str]] = {
            "HOST": os.getenv("HOST"),
            "GRPC_PORT": os.getenv("GRPC_PORT"),
            "LCD_PORT": os.getenv("LCD_PORT"),
            "CHAIN_ID": os.getenv("CHAIN_ID"),
        }
        for name, env_var in chain_config.items():
            if env_var is None: 
                raise Exception("\n".join([
                    f"Environment variable {name} is needed for devnet.", 
                    f"Please set {name} in your .env file."
                    ]))
        return cls(
            lcd_endpoint=f'http://{chain_config["HOST"]}.:{chain_config["LCD_PORT"]}',
            grpc_endpoint=f'{chain_config["HOST"]}:{chain_config["GRPC_PORT"]}',
            chain_id=chain_config["CHAIN_ID"],
            fee_denom='unibi',
            env="devnet",
        )

    @classmethod
    def testnet(cls) -> "Network":
        return cls(
            lcd_endpoint='https://lcd.testnet-3.nibiru.fi',
            grpc_endpoint='grpc.testnet-3.nibiru.fi',
            chain_id='nibiru-testnet-3',
            fee_denom='unibi',
            env='testnet',
        )

    @classmethod
    def mainnet(cls) -> "Network":
        raise NotImplementedError

    @classmethod
    def local(cls) -> "Network":
        return cls(
            lcd_endpoint='http://localhost:1317',
            grpc_endpoint='localhost:9090',
            chain_id='nibiru-localnet-0',
            fee_denom='unibi',
            env='local',
        )

    def string(self):
        return self.env



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import abc
import grpc
import oracle.v1beta1.query_pb2

class QueryStub:
    """Query defines the gRPC querier service."""
    def __init__(self, channel: grpc.Channel) -> None: ...
    ExchangeRate: grpc.UnaryUnaryMultiCallable[
        oracle.v1beta1.query_pb2.QueryExchangeRateRequest,
        oracle.v1beta1.query_pb2.QueryExchangeRateResponse]
    """ExchangeRate returns exchange rate of a pair"""

    ExchangeRates: grpc.UnaryUnaryMultiCallable[
        oracle.v1beta1.query_pb2.QueryExchangeRatesRequest,
        oracle.v1beta1.query_pb2.QueryExchangeRatesResponse]
    """ExchangeRates returns exchange rates of all pairs"""

    TobinTax: grpc.UnaryUnaryMultiCallable[
        oracle.v1beta1.query_pb2.QueryTobinTaxRequest,
        oracle.v1beta1.query_pb2.QueryTobinTaxResponse]
    """TobinTax returns tobin tax of a pair"""

    TobinTaxes: grpc.UnaryUnaryMultiCallable[
        oracle.v1beta1.query_pb2.QueryTobinTaxesRequest,
        oracle.v1beta1.query_pb2.QueryTobinTaxesResponse]
    """TobinTaxes returns tobin taxes of all pairs"""

    Actives: grpc.UnaryUnaryMultiCallable[
        oracle.v1beta1.query_pb2.QueryActivesRequest,
        oracle.v1beta1.query_pb2.QueryActivesResponse]
    """Actives returns all active pairs"""

    VoteTargets: grpc.UnaryUnaryMultiCallable[
        oracle.v1beta1.query_pb2.QueryVoteTargetsRequest,
        oracle.v1beta1.query_pb2.QueryVoteTargetsResponse]
    """VoteTargets returns all vote target for pairs"""

    FeederDelegation: grpc.UnaryUnaryMultiCallable[
        oracle.v1beta1.query_pb2.QueryFeederDelegationRequest,
        oracle.v1beta1.query_pb2.QueryFeederDelegationResponse]
    """FeederDelegation returns feeder delegation of a validator"""

    MissCounter: grpc.UnaryUnaryMultiCallable[
        oracle.v1beta1.query_pb2.QueryMissCounterRequest,
        oracle.v1beta1.query_pb2.QueryMissCounterResponse]
    """MissCounter returns oracle miss counter of a validator"""

    AggregatePrevote: grpc.UnaryUnaryMultiCallable[
        oracle.v1beta1.query_pb2.QueryAggregatePrevoteRequest,
        oracle.v1beta1.query_pb2.QueryAggregatePrevoteResponse]
    """AggregatePrevote returns an aggregate prevote of a validator"""

    AggregatePrevotes: grpc.UnaryUnaryMultiCallable[
        oracle.v1beta1.query_pb2.QueryAggregatePrevotesRequest,
        oracle.v1beta1.query_pb2.QueryAggregatePrevotesResponse]
    """AggregatePrevotes returns aggregate prevotes of all validators"""

    AggregateVote: grpc.UnaryUnaryMultiCallable[
        oracle.v1beta1.query_pb2.QueryAggregateVoteRequest,
        oracle.v1beta1.query_pb2.QueryAggregateVoteResponse]
    """AggregateVote returns an aggregate vote of a validator"""

    AggregateVotes: grpc.UnaryUnaryMultiCallable[
        oracle.v1beta1.query_pb2.QueryAggregateVotesRequest,
        oracle.v1beta1.query_pb2.QueryAggregateVotesResponse]
    """AggregateVotes returns aggregate votes of all validators"""

    Params: grpc.UnaryUnaryMultiCallable[
        oracle.v1beta1.query_pb2.QueryParamsRequest,
        oracle.v1beta1.query_pb2.QueryParamsResponse]
    """Params queries all parameters."""


class QueryServicer(metaclass=abc.ABCMeta):
    """Query defines the gRPC querier service."""
    @abc.abstractmethod
    def ExchangeRate(self,
        request: oracle.v1beta1.query_pb2.QueryExchangeRateRequest,
        context: grpc.ServicerContext,
    ) -> oracle.v1beta1.query_pb2.QueryExchangeRateResponse:
        """ExchangeRate returns exchange rate of a pair"""
        pass

    @abc.abstractmethod
    def ExchangeRates(self,
        request: oracle.v1beta1.query_pb2.QueryExchangeRatesRequest,
        context: grpc.ServicerContext,
    ) -> oracle.v1beta1.query_pb2.QueryExchangeRatesResponse:
        """ExchangeRates returns exchange rates of all pairs"""
        pass

    @abc.abstractmethod
    def TobinTax(self,
        request: oracle.v1beta1.query_pb2.QueryTobinTaxRequest,
        context: grpc.ServicerContext,
    ) -> oracle.v1beta1.query_pb2.QueryTobinTaxResponse:
        """TobinTax returns tobin tax of a pair"""
        pass

    @abc.abstractmethod
    def TobinTaxes(self,
        request: oracle.v1beta1.query_pb2.QueryTobinTaxesRequest,
        context: grpc.ServicerContext,
    ) -> oracle.v1beta1.query_pb2.QueryTobinTaxesResponse:
        """TobinTaxes returns tobin taxes of all pairs"""
        pass

    @abc.abstractmethod
    def Actives(self,
        request: oracle.v1beta1.query_pb2.QueryActivesRequest,
        context: grpc.ServicerContext,
    ) -> oracle.v1beta1.query_pb2.QueryActivesResponse:
        """Actives returns all active pairs"""
        pass

    @abc.abstractmethod
    def VoteTargets(self,
        request: oracle.v1beta1.query_pb2.QueryVoteTargetsRequest,
        context: grpc.ServicerContext,
    ) -> oracle.v1beta1.query_pb2.QueryVoteTargetsResponse:
        """VoteTargets returns all vote target for pairs"""
        pass

    @abc.abstractmethod
    def FeederDelegation(self,
        request: oracle.v1beta1.query_pb2.QueryFeederDelegationRequest,
        context: grpc.ServicerContext,
    ) -> oracle.v1beta1.query_pb2.QueryFeederDelegationResponse:
        """FeederDelegation returns feeder delegation of a validator"""
        pass

    @abc.abstractmethod
    def MissCounter(self,
        request: oracle.v1beta1.query_pb2.QueryMissCounterRequest,
        context: grpc.ServicerContext,
    ) -> oracle.v1beta1.query_pb2.QueryMissCounterResponse:
        """MissCounter returns oracle miss counter of a validator"""
        pass

    @abc.abstractmethod
    def AggregatePrevote(self,
        request: oracle.v1beta1.query_pb2.QueryAggregatePrevoteRequest,
        context: grpc.ServicerContext,
    ) -> oracle.v1beta1.query_pb2.QueryAggregatePrevoteResponse:
        """AggregatePrevote returns an aggregate prevote of a validator"""
        pass

    @abc.abstractmethod
    def AggregatePrevotes(self,
        request: oracle.v1beta1.query_pb2.QueryAggregatePrevotesRequest,
        context: grpc.ServicerContext,
    ) -> oracle.v1beta1.query_pb2.QueryAggregatePrevotesResponse:
        """AggregatePrevotes returns aggregate prevotes of all validators"""
        pass

    @abc.abstractmethod
    def AggregateVote(self,
        request: oracle.v1beta1.query_pb2.QueryAggregateVoteRequest,
        context: grpc.ServicerContext,
    ) -> oracle.v1beta1.query_pb2.QueryAggregateVoteResponse:
        """AggregateVote returns an aggregate vote of a validator"""
        pass

    @abc.abstractmethod
    def AggregateVotes(self,
        request: oracle.v1beta1.query_pb2.QueryAggregateVotesRequest,
        context: grpc.ServicerContext,
    ) -> oracle.v1beta1.query_pb2.QueryAggregateVotesResponse:
        """AggregateVotes returns aggregate votes of all validators"""
        pass

    @abc.abstractmethod
    def Params(self,
        request: oracle.v1beta1.query_pb2.QueryParamsRequest,
        context: grpc.ServicerContext,
    ) -> oracle.v1beta1.query_pb2.QueryParamsResponse:
        """Params queries all parameters."""
        pass


def add_QueryServicer_to_server(servicer: QueryServicer, server: grpc.Server) -> None: ...



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import cosmos.base.v1beta1.coin_pb2
import google.protobuf.descriptor
import google.protobuf.duration_pb2
import google.protobuf.internal.containers
import google.protobuf.message
import google.protobuf.timestamp_pb2
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class Lock(google.protobuf.message.Message):
    """Lock represents a users locked tokens for a period of time.
    It stores owner, duration, unlock time and the amount of coins locked.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    LOCK_ID_FIELD_NUMBER: builtins.int
    OWNER_FIELD_NUMBER: builtins.int
    DURATION_FIELD_NUMBER: builtins.int
    END_TIME_FIELD_NUMBER: builtins.int
    COINS_FIELD_NUMBER: builtins.int
    lock_id: builtins.int
    """unique autoincrementing numeric lock id"""

    owner: typing.Text
    """the user's address who owns the tokens that are locked"""

    @property
    def duration(self) -> google.protobuf.duration_pb2.Duration:
        """the duration that the lock is locked for"""
        pass
    @property
    def end_time(self) -> google.protobuf.timestamp_pb2.Timestamp:
        """when the lock was unlocked"""
        pass
    @property
    def coins(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.base.v1beta1.coin_pb2.Coin]:
        """the coins locked in this Lock"""
        pass
    def __init__(self,
        *,
        lock_id: builtins.int = ...,
        owner: typing.Text = ...,
        duration: typing.Optional[google.protobuf.duration_pb2.Duration] = ...,
        end_time: typing.Optional[google.protobuf.timestamp_pb2.Timestamp] = ...,
        coins: typing.Optional[typing.Iterable[cosmos.base.v1beta1.coin_pb2.Coin]] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["duration",b"duration","end_time",b"end_time"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["coins",b"coins","duration",b"duration","end_time",b"end_time","lock_id",b"lock_id","owner",b"owner"]) -> None: ...
global___Lock = Lock



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import cosmos.base.v1beta1.coin_pb2
import google.protobuf.descriptor
import google.protobuf.duration_pb2
import google.protobuf.internal.containers
import google.protobuf.message
import google.protobuf.timestamp_pb2
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class MsgLockTokens(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    OWNER_FIELD_NUMBER: builtins.int
    DURATION_FIELD_NUMBER: builtins.int
    COINS_FIELD_NUMBER: builtins.int
    owner: typing.Text
    @property
    def duration(self) -> google.protobuf.duration_pb2.Duration: ...
    @property
    def coins(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.base.v1beta1.coin_pb2.Coin]: ...
    def __init__(self,
        *,
        owner: typing.Text = ...,
        duration: typing.Optional[google.protobuf.duration_pb2.Duration] = ...,
        coins: typing.Optional[typing.Iterable[cosmos.base.v1beta1.coin_pb2.Coin]] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["duration",b"duration"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["coins",b"coins","duration",b"duration","owner",b"owner"]) -> None: ...
global___MsgLockTokens = MsgLockTokens

class MsgLockTokensResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    LOCK_ID_FIELD_NUMBER: builtins.int
    lock_id: builtins.int
    def __init__(self,
        *,
        lock_id: builtins.int = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["lock_id",b"lock_id"]) -> None: ...
global___MsgLockTokensResponse = MsgLockTokensResponse

class MsgInitiateUnlock(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    OWNER_FIELD_NUMBER: builtins.int
    LOCK_ID_FIELD_NUMBER: builtins.int
    owner: typing.Text
    lock_id: builtins.int
    def __init__(self,
        *,
        owner: typing.Text = ...,
        lock_id: builtins.int = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["lock_id",b"lock_id","owner",b"owner"]) -> None: ...
global___MsgInitiateUnlock = MsgInitiateUnlock

class MsgInitiateUnlockResponse(google.protobuf.message.Message):
    """TODO(mercilex): maybe report unlock time"""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    def __init__(self,
        ) -> None: ...
global___MsgInitiateUnlockResponse = MsgInitiateUnlockResponse

class MsgUnlock(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    OWNER_FIELD_NUMBER: builtins.int
    LOCK_ID_FIELD_NUMBER: builtins.int
    owner: typing.Text
    lock_id: builtins.int
    def __init__(self,
        *,
        owner: typing.Text = ...,
        lock_id: builtins.int = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["lock_id",b"lock_id","owner",b"owner"]) -> None: ...
global___MsgUnlock = MsgUnlock

class MsgUnlockResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    def __init__(self,
        ) -> None: ...
global___MsgUnlockResponse = MsgUnlockResponse

class EventLock(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    LOCK_ID_FIELD_NUMBER: builtins.int
    OWNER_FIELD_NUMBER: builtins.int
    DURATION_FIELD_NUMBER: builtins.int
    COINS_FIELD_NUMBER: builtins.int
    lock_id: builtins.int
    owner: typing.Text
    @property
    def duration(self) -> google.protobuf.duration_pb2.Duration: ...
    @property
    def coins(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.base.v1beta1.coin_pb2.Coin]: ...
    def __init__(self,
        *,
        lock_id: builtins.int = ...,
        owner: typing.Text = ...,
        duration: typing.Optional[google.protobuf.duration_pb2.Duration] = ...,
        coins: typing.Optional[typing.Iterable[cosmos.base.v1beta1.coin_pb2.Coin]] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["duration",b"duration"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["coins",b"coins","duration",b"duration","lock_id",b"lock_id","owner",b"owner"]) -> None: ...
global___EventLock = EventLock

class EventUnlockInitiated(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    LOCK_ID_FIELD_NUMBER: builtins.int
    OWNER_FIELD_NUMBER: builtins.int
    COINS_FIELD_NUMBER: builtins.int
    UNLOCKING_AT_FIELD_NUMBER: builtins.int
    lock_id: builtins.int
    owner: typing.Text
    @property
    def coins(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.base.v1beta1.coin_pb2.Coin]: ...
    @property
    def unlocking_at(self) -> google.protobuf.timestamp_pb2.Timestamp: ...
    def __init__(self,
        *,
        lock_id: builtins.int = ...,
        owner: typing.Text = ...,
        coins: typing.Optional[typing.Iterable[cosmos.base.v1beta1.coin_pb2.Coin]] = ...,
        unlocking_at: typing.Optional[google.protobuf.timestamp_pb2.Timestamp] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["unlocking_at",b"unlocking_at"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["coins",b"coins","lock_id",b"lock_id","owner",b"owner","unlocking_at",b"unlocking_at"]) -> None: ...
global___EventUnlockInitiated = EventUnlockInitiated

class EventUnlock(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    LOCK_ID_FIELD_NUMBER: builtins.int
    OWNER_FIELD_NUMBER: builtins.int
    COINS_FIELD_NUMBER: builtins.int
    lock_id: builtins.int
    owner: typing.Text
    @property
    def coins(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.base.v1beta1.coin_pb2.Coin]: ...
    def __init__(self,
        *,
        lock_id: builtins.int = ...,
        owner: typing.Text = ...,
        coins: typing.Optional[typing.Iterable[cosmos.base.v1beta1.coin_pb2.Coin]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["coins",b"coins","lock_id",b"lock_id","owner",b"owner"]) -> None: ...
global___EventUnlock = EventUnlock



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.message
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class CreatePoolProposal(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    TITLE_FIELD_NUMBER: builtins.int
    DESCRIPTION_FIELD_NUMBER: builtins.int
    PAIR_FIELD_NUMBER: builtins.int
    TRADE_LIMIT_RATIO_FIELD_NUMBER: builtins.int
    QUOTE_ASSET_RESERVE_FIELD_NUMBER: builtins.int
    BASE_ASSET_RESERVE_FIELD_NUMBER: builtins.int
    FLUCTUATION_LIMIT_RATIO_FIELD_NUMBER: builtins.int
    MAX_ORACLE_SPREAD_RATIO_FIELD_NUMBER: builtins.int
    MAINTENANCE_MARGIN_RATIO_FIELD_NUMBER: builtins.int
    MAX_LEVERAGE_FIELD_NUMBER: builtins.int
    title: typing.Text
    description: typing.Text
    pair: typing.Text
    """pair represents the pair of the vpool."""

    trade_limit_ratio: typing.Text
    """trade_limit_ratio represents the limit on trading amounts."""

    quote_asset_reserve: typing.Text
    """quote_asset_reserve is the amount of quote asset the pool will be initialized with."""

    base_asset_reserve: typing.Text
    """base_asset_reserve is the amount of base asset the pool will be initialized with."""

    fluctuation_limit_ratio: typing.Text
    """fluctuation_limit_ratio represents the maximum price
    percentage difference a trade can create on the pool.
    """

    max_oracle_spread_ratio: typing.Text
    """max_oracle_spread_ratio represents the maximum price percentage
    difference that can exist between oracle price and vpool prices after a trade.
    """

    maintenance_margin_ratio: typing.Text
    """maintenance_margin_ratio"""

    max_leverage: typing.Text
    """max_leverage"""

    def __init__(self,
        *,
        title: typing.Text = ...,
        description: typing.Text = ...,
        pair: typing.Text = ...,
        trade_limit_ratio: typing.Text = ...,
        quote_asset_reserve: typing.Text = ...,
        base_asset_reserve: typing.Text = ...,
        fluctuation_limit_ratio: typing.Text = ...,
        max_oracle_spread_ratio: typing.Text = ...,
        maintenance_margin_ratio: typing.Text = ...,
        max_leverage: typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["base_asset_reserve",b"base_asset_reserve","description",b"description","fluctuation_limit_ratio",b"fluctuation_limit_ratio","maintenance_margin_ratio",b"maintenance_margin_ratio","max_leverage",b"max_leverage","max_oracle_spread_ratio",b"max_oracle_spread_ratio","pair",b"pair","quote_asset_reserve",b"quote_asset_reserve","title",b"title","trade_limit_ratio",b"trade_limit_ratio"]) -> None: ...
global___CreatePoolProposal = CreatePoolProposal



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import cosmos.base.query.v1beta1.pagination_pb2
import cosmos.staking.v1beta1.staking_pb2
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class QueryValidatorsRequest(google.protobuf.message.Message):
    """QueryValidatorsRequest is request type for Query/Validators RPC method."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    STATUS_FIELD_NUMBER: builtins.int
    PAGINATION_FIELD_NUMBER: builtins.int
    status: typing.Text
    """status enables to query for validators matching a given status."""

    @property
    def pagination(self) -> cosmos.base.query.v1beta1.pagination_pb2.PageRequest:
        """pagination defines an optional pagination for the request."""
        pass
    def __init__(self,
        *,
        status: typing.Text = ...,
        pagination: typing.Optional[cosmos.base.query.v1beta1.pagination_pb2.PageRequest] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["pagination",b"pagination"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["pagination",b"pagination","status",b"status"]) -> None: ...
global___QueryValidatorsRequest = QueryValidatorsRequest

class QueryValidatorsResponse(google.protobuf.message.Message):
    """QueryValidatorsResponse is response type for the Query/Validators RPC method"""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    VALIDATORS_FIELD_NUMBER: builtins.int
    PAGINATION_FIELD_NUMBER: builtins.int
    @property
    def validators(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.staking.v1beta1.staking_pb2.Validator]:
        """validators contains all the queried validators."""
        pass
    @property
    def pagination(self) -> cosmos.base.query.v1beta1.pagination_pb2.PageResponse:
        """pagination defines the pagination in the response."""
        pass
    def __init__(self,
        *,
        validators: typing.Optional[typing.Iterable[cosmos.staking.v1beta1.staking_pb2.Validator]] = ...,
        pagination: typing.Optional[cosmos.base.query.v1beta1.pagination_pb2.PageResponse] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["pagination",b"pagination"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["pagination",b"pagination","validators",b"validators"]) -> None: ...
global___QueryValidatorsResponse = QueryValidatorsResponse

class QueryValidatorRequest(google.protobuf.message.Message):
    """QueryValidatorRequest is response type for the Query/Validator RPC method"""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    VALIDATOR_ADDR_FIELD_NUMBER: builtins.int
    validator_addr: typing.Text
    """validator_addr defines the validator address to query for."""

    def __init__(self,
        *,
        validator_addr: typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["validator_addr",b"validator_addr"]) -> None: ...
global___QueryValidatorRequest = QueryValidatorRequest

class QueryValidatorResponse(google.protobuf.message.Message):
    """QueryValidatorResponse is response type for the Query/Validator RPC method"""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    VALIDATOR_FIELD_NUMBER: builtins.int
    @property
    def validator(self) -> cosmos.staking.v1beta1.staking_pb2.Validator:
        """validator defines the the validator info."""
        pass
    def __init__(self,
        *,
        validator: typing.Optional[cosmos.staking.v1beta1.staking_pb2.Validator] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["validator",b"validator"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["validator",b"validator"]) -> None: ...
global___QueryValidatorResponse = QueryValidatorResponse

class QueryValidatorDelegationsRequest(google.protobuf.message.Message):
    """QueryValidatorDelegationsRequest is request type for the
    Query/ValidatorDelegations RPC method
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    VALIDATOR_ADDR_FIELD_NUMBER: builtins.int
    PAGINATION_FIELD_NUMBER: builtins.int
    validator_addr: typing.Text
    """validator_addr defines the validator address to query for."""

    @property
    def pagination(self) -> cosmos.base.query.v1beta1.pagination_pb2.PageRequest:
        """pagination defines an optional pagination for the request."""
        pass
    def __init__(self,
        *,
        validator_addr: typing.Text = ...,
        pagination: typing.Optional[cosmos.base.query.v1beta1.pagination_pb2.PageRequest] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["pagination",b"pagination"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["pagination",b"pagination","validator_addr",b"validator_addr"]) -> None: ...
global___QueryValidatorDelegationsRequest = QueryValidatorDelegationsRequest

class QueryValidatorDelegationsResponse(google.protobuf.message.Message):
    """QueryValidatorDelegationsResponse is response type for the
    Query/ValidatorDelegations RPC method
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    DELEGATION_RESPONSES_FIELD_NUMBER: builtins.int
    PAGINATION_FIELD_NUMBER: builtins.int
    @property
    def delegation_responses(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.staking.v1beta1.staking_pb2.DelegationResponse]: ...
    @property
    def pagination(self) -> cosmos.base.query.v1beta1.pagination_pb2.PageResponse:
        """pagination defines the pagination in the response."""
        pass
    def __init__(self,
        *,
        delegation_responses: typing.Optional[typing.Iterable[cosmos.staking.v1beta1.staking_pb2.DelegationResponse]] = ...,
        pagination: typing.Optional[cosmos.base.query.v1beta1.pagination_pb2.PageResponse] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["pagination",b"pagination"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["delegation_responses",b"delegation_responses","pagination",b"pagination"]) -> None: ...
global___QueryValidatorDelegationsResponse = QueryValidatorDelegationsResponse

class QueryValidatorUnbondingDelegationsRequest(google.protobuf.message.Message):
    """QueryValidatorUnbondingDelegationsRequest is required type for the
    Query/ValidatorUnbondingDelegations RPC method
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    VALIDATOR_ADDR_FIELD_NUMBER: builtins.int
    PAGINATION_FIELD_NUMBER: builtins.int
    validator_addr: typing.Text
    """validator_addr defines the validator address to query for."""

    @property
    def pagination(self) -> cosmos.base.query.v1beta1.pagination_pb2.PageRequest:
        """pagination defines an optional pagination for the request."""
        pass
    def __init__(self,
        *,
        validator_addr: typing.Text = ...,
        pagination: typing.Optional[cosmos.base.query.v1beta1.pagination_pb2.PageRequest] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["pagination",b"pagination"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["pagination",b"pagination","validator_addr",b"validator_addr"]) -> None: ...
global___QueryValidatorUnbondingDelegationsRequest = QueryValidatorUnbondingDelegationsRequest

class QueryValidatorUnbondingDelegationsResponse(google.protobuf.message.Message):
    """QueryValidatorUnbondingDelegationsResponse is response type for the
    Query/ValidatorUnbondingDelegations RPC method.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    UNBONDING_RESPONSES_FIELD_NUMBER: builtins.int
    PAGINATION_FIELD_NUMBER: builtins.int
    @property
    def unbonding_responses(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.staking.v1beta1.staking_pb2.UnbondingDelegation]: ...
    @property
    def pagination(self) -> cosmos.base.query.v1beta1.pagination_pb2.PageResponse:
        """pagination defines the pagination in the response."""
        pass
    def __init__(self,
        *,
        unbonding_responses: typing.Optional[typing.Iterable[cosmos.staking.v1beta1.staking_pb2.UnbondingDelegation]] = ...,
        pagination: typing.Optional[cosmos.base.query.v1beta1.pagination_pb2.PageResponse] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["pagination",b"pagination"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["pagination",b"pagination","unbonding_responses",b"unbonding_responses"]) -> None: ...
global___QueryValidatorUnbondingDelegationsResponse = QueryValidatorUnbondingDelegationsResponse

class QueryDelegationRequest(google.protobuf.message.Message):
    """QueryDelegationRequest is request type for the Query/Delegation RPC method."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    DELEGATOR_ADDR_FIELD_NUMBER: builtins.int
    VALIDATOR_ADDR_FIELD_NUMBER: builtins.int
    delegator_addr: typing.Text
    """delegator_addr defines the delegator address to query for."""

    validator_addr: typing.Text
    """validator_addr defines the validator address to query for."""

    def __init__(self,
        *,
        delegator_addr: typing.Text = ...,
        validator_addr: typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["delegator_addr",b"delegator_addr","validator_addr",b"validator_addr"]) -> None: ...
global___QueryDelegationRequest = QueryDelegationRequest

class QueryDelegationResponse(google.protobuf.message.Message):
    """QueryDelegationResponse is response type for the Query/Delegation RPC method."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    DELEGATION_RESPONSE_FIELD_NUMBER: builtins.int
    @property
    def delegation_response(self) -> cosmos.staking.v1beta1.staking_pb2.DelegationResponse:
        """delegation_responses defines the delegation info of a delegation."""
        pass
    def __init__(self,
        *,
        delegation_response: typing.Optional[cosmos.staking.v1beta1.staking_pb2.DelegationResponse] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["delegation_response",b"delegation_response"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["delegation_response",b"delegation_response"]) -> None: ...
global___QueryDelegationResponse = QueryDelegationResponse

class QueryUnbondingDelegationRequest(google.protobuf.message.Message):
    """QueryUnbondingDelegationRequest is request type for the
    Query/UnbondingDelegation RPC method.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    DELEGATOR_ADDR_FIELD_NUMBER: builtins.int
    VALIDATOR_ADDR_FIELD_NUMBER: builtins.int
    delegator_addr: typing.Text
    """delegator_addr defines the delegator address to query for."""

    validator_addr: typing.Text
    """validator_addr defines the validator address to query for."""

    def __init__(self,
        *,
        delegator_addr: typing.Text = ...,
        validator_addr: typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["delegator_addr",b"delegator_addr","validator_addr",b"validator_addr"]) -> None: ...
global___QueryUnbondingDelegationRequest = QueryUnbondingDelegationRequest

class QueryUnbondingDelegationResponse(google.protobuf.message.Message):
    """QueryDelegationResponse is response type for the Query/UnbondingDelegation
    RPC method.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    UNBOND_FIELD_NUMBER: builtins.int
    @property
    def unbond(self) -> cosmos.staking.v1beta1.staking_pb2.UnbondingDelegation:
        """unbond defines the unbonding information of a delegation."""
        pass
    def __init__(self,
        *,
        unbond: typing.Optional[cosmos.staking.v1beta1.staking_pb2.UnbondingDelegation] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["unbond",b"unbond"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["unbond",b"unbond"]) -> None: ...
global___QueryUnbondingDelegationResponse = QueryUnbondingDelegationResponse

class QueryDelegatorDelegationsRequest(google.protobuf.message.Message):
    """QueryDelegatorDelegationsRequest is request type for the
    Query/DelegatorDelegations RPC method.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    DELEGATOR_ADDR_FIELD_NUMBER: builtins.int
    PAGINATION_FIELD_NUMBER: builtins.int
    delegator_addr: typing.Text
    """delegator_addr defines the delegator address to query for."""

    @property
    def pagination(self) -> cosmos.base.query.v1beta1.pagination_pb2.PageRequest:
        """pagination defines an optional pagination for the request."""
        pass
    def __init__(self,
        *,
        delegator_addr: typing.Text = ...,
        pagination: typing.Optional[cosmos.base.query.v1beta1.pagination_pb2.PageRequest] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["pagination",b"pagination"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["delegator_addr",b"delegator_addr","pagination",b"pagination"]) -> None: ...
global___QueryDelegatorDelegationsRequest = QueryDelegatorDelegationsRequest

class QueryDelegatorDelegationsResponse(google.protobuf.message.Message):
    """QueryDelegatorDelegationsResponse is response type for the
    Query/DelegatorDelegations RPC method.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    DELEGATION_RESPONSES_FIELD_NUMBER: builtins.int
    PAGINATION_FIELD_NUMBER: builtins.int
    @property
    def delegation_responses(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.staking.v1beta1.staking_pb2.DelegationResponse]:
        """delegation_responses defines all the delegations' info of a delegator."""
        pass
    @property
    def pagination(self) -> cosmos.base.query.v1beta1.pagination_pb2.PageResponse:
        """pagination defines the pagination in the response."""
        pass
    def __init__(self,
        *,
        delegation_responses: typing.Optional[typing.Iterable[cosmos.staking.v1beta1.staking_pb2.DelegationResponse]] = ...,
        pagination: typing.Optional[cosmos.base.query.v1beta1.pagination_pb2.PageResponse] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["pagination",b"pagination"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["delegation_responses",b"delegation_responses","pagination",b"pagination"]) -> None: ...
global___QueryDelegatorDelegationsResponse = QueryDelegatorDelegationsResponse

class QueryDelegatorUnbondingDelegationsRequest(google.protobuf.message.Message):
    """QueryDelegatorUnbondingDelegationsRequest is request type for the
    Query/DelegatorUnbondingDelegations RPC method.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    DELEGATOR_ADDR_FIELD_NUMBER: builtins.int
    PAGINATION_FIELD_NUMBER: builtins.int
    delegator_addr: typing.Text
    """delegator_addr defines the delegator address to query for."""

    @property
    def pagination(self) -> cosmos.base.query.v1beta1.pagination_pb2.PageRequest:
        """pagination defines an optional pagination for the request."""
        pass
    def __init__(self,
        *,
        delegator_addr: typing.Text = ...,
        pagination: typing.Optional[cosmos.base.query.v1beta1.pagination_pb2.PageRequest] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["pagination",b"pagination"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["delegator_addr",b"delegator_addr","pagination",b"pagination"]) -> None: ...
global___QueryDelegatorUnbondingDelegationsRequest = QueryDelegatorUnbondingDelegationsRequest

class QueryDelegatorUnbondingDelegationsResponse(google.protobuf.message.Message):
    """QueryUnbondingDelegatorDelegationsResponse is response type for the
    Query/UnbondingDelegatorDelegations RPC method.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    UNBONDING_RESPONSES_FIELD_NUMBER: builtins.int
    PAGINATION_FIELD_NUMBER: builtins.int
    @property
    def unbonding_responses(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.staking.v1beta1.staking_pb2.UnbondingDelegation]: ...
    @property
    def pagination(self) -> cosmos.base.query.v1beta1.pagination_pb2.PageResponse:
        """pagination defines the pagination in the response."""
        pass
    def __init__(self,
        *,
        unbonding_responses: typing.Optional[typing.Iterable[cosmos.staking.v1beta1.staking_pb2.UnbondingDelegation]] = ...,
        pagination: typing.Optional[cosmos.base.query.v1beta1.pagination_pb2.PageResponse] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["pagination",b"pagination"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["pagination",b"pagination","unbonding_responses",b"unbonding_responses"]) -> None: ...
global___QueryDelegatorUnbondingDelegationsResponse = QueryDelegatorUnbondingDelegationsResponse

class QueryRedelegationsRequest(google.protobuf.message.Message):
    """QueryRedelegationsRequest is request type for the Query/Redelegations RPC
    method.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    DELEGATOR_ADDR_FIELD_NUMBER: builtins.int
    SRC_VALIDATOR_ADDR_FIELD_NUMBER: builtins.int
    DST_VALIDATOR_ADDR_FIELD_NUMBER: builtins.int
    PAGINATION_FIELD_NUMBER: builtins.int
    delegator_addr: typing.Text
    """delegator_addr defines the delegator address to query for."""

    src_validator_addr: typing.Text
    """src_validator_addr defines the validator address to redelegate from."""

    dst_validator_addr: typing.Text
    """dst_validator_addr defines the validator address to redelegate to."""

    @property
    def pagination(self) -> cosmos.base.query.v1beta1.pagination_pb2.PageRequest:
        """pagination defines an optional pagination for the request."""
        pass
    def __init__(self,
        *,
        delegator_addr: typing.Text = ...,
        src_validator_addr: typing.Text = ...,
        dst_validator_addr: typing.Text = ...,
        pagination: typing.Optional[cosmos.base.query.v1beta1.pagination_pb2.PageRequest] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["pagination",b"pagination"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["delegator_addr",b"delegator_addr","dst_validator_addr",b"dst_validator_addr","pagination",b"pagination","src_validator_addr",b"src_validator_addr"]) -> None: ...
global___QueryRedelegationsRequest = QueryRedelegationsRequest

class QueryRedelegationsResponse(google.protobuf.message.Message):
    """QueryRedelegationsResponse is response type for the Query/Redelegations RPC
    method.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    REDELEGATION_RESPONSES_FIELD_NUMBER: builtins.int
    PAGINATION_FIELD_NUMBER: builtins.int
    @property
    def redelegation_responses(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.staking.v1beta1.staking_pb2.RedelegationResponse]: ...
    @property
    def pagination(self) -> cosmos.base.query.v1beta1.pagination_pb2.PageResponse:
        """pagination defines the pagination in the response."""
        pass
    def __init__(self,
        *,
        redelegation_responses: typing.Optional[typing.Iterable[cosmos.staking.v1beta1.staking_pb2.RedelegationResponse]] = ...,
        pagination: typing.Optional[cosmos.base.query.v1beta1.pagination_pb2.PageResponse] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["pagination",b"pagination"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["pagination",b"pagination","redelegation_responses",b"redelegation_responses"]) -> None: ...
global___QueryRedelegationsResponse = QueryRedelegationsResponse

class QueryDelegatorValidatorsRequest(google.protobuf.message.Message):
    """QueryDelegatorValidatorsRequest is request type for the
    Query/DelegatorValidators RPC method.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    DELEGATOR_ADDR_FIELD_NUMBER: builtins.int
    PAGINATION_FIELD_NUMBER: builtins.int
    delegator_addr: typing.Text
    """delegator_addr defines the delegator address to query for."""

    @property
    def pagination(self) -> cosmos.base.query.v1beta1.pagination_pb2.PageRequest:
        """pagination defines an optional pagination for the request."""
        pass
    def __init__(self,
        *,
        delegator_addr: typing.Text = ...,
        pagination: typing.Optional[cosmos.base.query.v1beta1.pagination_pb2.PageRequest] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["pagination",b"pagination"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["delegator_addr",b"delegator_addr","pagination",b"pagination"]) -> None: ...
global___QueryDelegatorValidatorsRequest = QueryDelegatorValidatorsRequest

class QueryDelegatorValidatorsResponse(google.protobuf.message.Message):
    """QueryDelegatorValidatorsResponse is response type for the
    Query/DelegatorValidators RPC method.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    VALIDATORS_FIELD_NUMBER: builtins.int
    PAGINATION_FIELD_NUMBER: builtins.int
    @property
    def validators(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.staking.v1beta1.staking_pb2.Validator]:
        """validators defines the the validators' info of a delegator."""
        pass
    @property
    def pagination(self) -> cosmos.base.query.v1beta1.pagination_pb2.PageResponse:
        """pagination defines the pagination in the response."""
        pass
    def __init__(self,
        *,
        validators: typing.Optional[typing.Iterable[cosmos.staking.v1beta1.staking_pb2.Validator]] = ...,
        pagination: typing.Optional[cosmos.base.query.v1beta1.pagination_pb2.PageResponse] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["pagination",b"pagination"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["pagination",b"pagination","validators",b"validators"]) -> None: ...
global___QueryDelegatorValidatorsResponse = QueryDelegatorValidatorsResponse

class QueryDelegatorValidatorRequest(google.protobuf.message.Message):
    """QueryDelegatorValidatorRequest is request type for the
    Query/DelegatorValidator RPC method.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    DELEGATOR_ADDR_FIELD_NUMBER: builtins.int
    VALIDATOR_ADDR_FIELD_NUMBER: builtins.int
    delegator_addr: typing.Text
    """delegator_addr defines the delegator address to query for."""

    validator_addr: typing.Text
    """validator_addr defines the validator address to query for."""

    def __init__(self,
        *,
        delegator_addr: typing.Text = ...,
        validator_addr: typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["delegator_addr",b"delegator_addr","validator_addr",b"validator_addr"]) -> None: ...
global___QueryDelegatorValidatorRequest = QueryDelegatorValidatorRequest

class QueryDelegatorValidatorResponse(google.protobuf.message.Message):
    """QueryDelegatorValidatorResponse response type for the
    Query/DelegatorValidator RPC method.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    VALIDATOR_FIELD_NUMBER: builtins.int
    @property
    def validator(self) -> cosmos.staking.v1beta1.staking_pb2.Validator:
        """validator defines the the validator info."""
        pass
    def __init__(self,
        *,
        validator: typing.Optional[cosmos.staking.v1beta1.staking_pb2.Validator] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["validator",b"validator"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["validator",b"validator"]) -> None: ...
global___QueryDelegatorValidatorResponse = QueryDelegatorValidatorResponse

class QueryHistoricalInfoRequest(google.protobuf.message.Message):
    """QueryHistoricalInfoRequest is request type for the Query/HistoricalInfo RPC
    method.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    HEIGHT_FIELD_NUMBER: builtins.int
    height: builtins.int
    """height defines at which height to query the historical info."""

    def __init__(self,
        *,
        height: builtins.int = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["height",b"height"]) -> None: ...
global___QueryHistoricalInfoRequest = QueryHistoricalInfoRequest

class QueryHistoricalInfoResponse(google.protobuf.message.Message):
    """QueryHistoricalInfoResponse is response type for the Query/HistoricalInfo RPC
    method.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    HIST_FIELD_NUMBER: builtins.int
    @property
    def hist(self) -> cosmos.staking.v1beta1.staking_pb2.HistoricalInfo:
        """hist defines the historical info at the given height."""
        pass
    def __init__(self,
        *,
        hist: typing.Optional[cosmos.staking.v1beta1.staking_pb2.HistoricalInfo] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["hist",b"hist"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["hist",b"hist"]) -> None: ...
global___QueryHistoricalInfoResponse = QueryHistoricalInfoResponse

class QueryPoolRequest(google.protobuf.message.Message):
    """QueryPoolRequest is request type for the Query/Pool RPC method."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    def __init__(self,
        ) -> None: ...
global___QueryPoolRequest = QueryPoolRequest

class QueryPoolResponse(google.protobuf.message.Message):
    """QueryPoolResponse is response type for the Query/Pool RPC method."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    POOL_FIELD_NUMBER: builtins.int
    @property
    def pool(self) -> cosmos.staking.v1beta1.staking_pb2.Pool:
        """pool defines the pool info."""
        pass
    def __init__(self,
        *,
        pool: typing.Optional[cosmos.staking.v1beta1.staking_pb2.Pool] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["pool",b"pool"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["pool",b"pool"]) -> None: ...
global___QueryPoolResponse = QueryPoolResponse

class QueryParamsRequest(google.protobuf.message.Message):
    """QueryParamsRequest is request type for the Query/Params RPC method."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    def __init__(self,
        ) -> None: ...
global___QueryParamsRequest = QueryParamsRequest

class QueryParamsResponse(google.protobuf.message.Message):
    """QueryParamsResponse is response type for the Query/Params RPC method."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    PARAMS_FIELD_NUMBER: builtins.int
    @property
    def params(self) -> cosmos.staking.v1beta1.staking_pb2.Params:
        """params holds all the parameters of this module."""
        pass
    def __init__(self,
        *,
        params: typing.Optional[cosmos.staking.v1beta1.staking_pb2.Params] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["params",b"params"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["params",b"params"]) -> None: ...
global___QueryParamsResponse = QueryParamsResponse



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.message
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class StoreKVPair(google.protobuf.message.Message):
    """StoreKVPair is a KVStore KVPair used for listening to state changes (Sets and Deletes)
    It optionally includes the StoreKey for the originating KVStore and a Boolean flag to distinguish between Sets and
    Deletes

    Since: cosmos-sdk 0.43
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    STORE_KEY_FIELD_NUMBER: builtins.int
    DELETE_FIELD_NUMBER: builtins.int
    KEY_FIELD_NUMBER: builtins.int
    VALUE_FIELD_NUMBER: builtins.int
    store_key: typing.Text
    """the store key for the KVStore this pair originates from"""

    delete: builtins.bool
    """true indicates a delete operation, false indicates a set operation"""

    key: builtins.bytes
    value: builtins.bytes
    def __init__(self,
        *,
        store_key: typing.Text = ...,
        delete: builtins.bool = ...,
        key: builtins.bytes = ...,
        value: builtins.bytes = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["delete",b"delete","key",b"key","store_key",b"store_key","value",b"value"]) -> None: ...
global___StoreKVPair = StoreKVPair



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: vpool/v1/gov.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from gogoproto import gogo_pb2 as gogoproto_dot_gogo__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x12vpool/v1/gov.proto\x12\x0fnibiru.vpool.v1\x1a\x14gogoproto/gogo.proto\"\xe4\x04\n\x12\x43reatePoolProposal\x12\r\n\x05title\x18\x01 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x02 \x01(\t\x12\x0c\n\x04pair\x18\x03 \x01(\t\x12I\n\x11trade_limit_ratio\x18\x04 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x12K\n\x13quote_asset_reserve\x18\x05 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x12J\n\x12\x62\x61se_asset_reserve\x18\x06 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x12O\n\x17\x66luctuation_limit_ratio\x18\x07 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x12O\n\x17max_oracle_spread_ratio\x18\x08 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x12P\n\x18maintenance_margin_ratio\x18\t \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x12\x44\n\x0cmax_leverage\x18\n \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x42-Z+github.com/NibiruChain/nibiru/x/vpool/typesb\x06proto3')



_CREATEPOOLPROPOSAL = DESCRIPTOR.message_types_by_name['CreatePoolProposal']
CreatePoolProposal = _reflection.GeneratedProtocolMessageType('CreatePoolProposal', (_message.Message,), {
  'DESCRIPTOR' : _CREATEPOOLPROPOSAL,
  '__module__' : 'vpool.v1.gov_pb2'
  # @@protoc_insertion_point(class_scope:nibiru.vpool.v1.CreatePoolProposal)
  })
_sym_db.RegisterMessage(CreatePoolProposal)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z+github.com/NibiruChain/nibiru/x/vpool/types'
  _CREATEPOOLPROPOSAL.fields_by_name['trade_limit_ratio']._options = None
  _CREATEPOOLPROPOSAL.fields_by_name['trade_limit_ratio']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _CREATEPOOLPROPOSAL.fields_by_name['quote_asset_reserve']._options = None
  _CREATEPOOLPROPOSAL.fields_by_name['quote_asset_reserve']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _CREATEPOOLPROPOSAL.fields_by_name['base_asset_reserve']._options = None
  _CREATEPOOLPROPOSAL.fields_by_name['base_asset_reserve']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _CREATEPOOLPROPOSAL.fields_by_name['fluctuation_limit_ratio']._options = None
  _CREATEPOOLPROPOSAL.fields_by_name['fluctuation_limit_ratio']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _CREATEPOOLPROPOSAL.fields_by_name['max_oracle_spread_ratio']._options = None
  _CREATEPOOLPROPOSAL.fields_by_name['max_oracle_spread_ratio']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _CREATEPOOLPROPOSAL.fields_by_name['maintenance_margin_ratio']._options = None
  _CREATEPOOLPROPOSAL.fields_by_name['maintenance_margin_ratio']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _CREATEPOOLPROPOSAL.fields_by_name['max_leverage']._options = None
  _CREATEPOOLPROPOSAL.fields_by_name['max_leverage']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _CREATEPOOLPROPOSAL._serialized_start=62
  _CREATEPOOLPROPOSAL._serialized_end=674
# @@protoc_insertion_point(module_scope)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import cosmos.base.query.v1beta1.pagination_pb2
import cosmos.base.v1beta1.coin_pb2
import cosmos.distribution.v1beta1.distribution_pb2
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class QueryParamsRequest(google.protobuf.message.Message):
    """QueryParamsRequest is the request type for the Query/Params RPC method."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    def __init__(self,
        ) -> None: ...
global___QueryParamsRequest = QueryParamsRequest

class QueryParamsResponse(google.protobuf.message.Message):
    """QueryParamsResponse is the response type for the Query/Params RPC method."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    PARAMS_FIELD_NUMBER: builtins.int
    @property
    def params(self) -> cosmos.distribution.v1beta1.distribution_pb2.Params:
        """params defines the parameters of the module."""
        pass
    def __init__(self,
        *,
        params: typing.Optional[cosmos.distribution.v1beta1.distribution_pb2.Params] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["params",b"params"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["params",b"params"]) -> None: ...
global___QueryParamsResponse = QueryParamsResponse

class QueryValidatorOutstandingRewardsRequest(google.protobuf.message.Message):
    """QueryValidatorOutstandingRewardsRequest is the request type for the
    Query/ValidatorOutstandingRewards RPC method.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    VALIDATOR_ADDRESS_FIELD_NUMBER: builtins.int
    validator_address: typing.Text
    """validator_address defines the validator address to query for."""

    def __init__(self,
        *,
        validator_address: typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["validator_address",b"validator_address"]) -> None: ...
global___QueryValidatorOutstandingRewardsRequest = QueryValidatorOutstandingRewardsRequest

class QueryValidatorOutstandingRewardsResponse(google.protobuf.message.Message):
    """QueryValidatorOutstandingRewardsResponse is the response type for the
    Query/ValidatorOutstandingRewards RPC method.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    REWARDS_FIELD_NUMBER: builtins.int
    @property
    def rewards(self) -> cosmos.distribution.v1beta1.distribution_pb2.ValidatorOutstandingRewards: ...
    def __init__(self,
        *,
        rewards: typing.Optional[cosmos.distribution.v1beta1.distribution_pb2.ValidatorOutstandingRewards] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["rewards",b"rewards"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["rewards",b"rewards"]) -> None: ...
global___QueryValidatorOutstandingRewardsResponse = QueryValidatorOutstandingRewardsResponse

class QueryValidatorCommissionRequest(google.protobuf.message.Message):
    """QueryValidatorCommissionRequest is the request type for the
    Query/ValidatorCommission RPC method
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    VALIDATOR_ADDRESS_FIELD_NUMBER: builtins.int
    validator_address: typing.Text
    """validator_address defines the validator address to query for."""

    def __init__(self,
        *,
        validator_address: typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["validator_address",b"validator_address"]) -> None: ...
global___QueryValidatorCommissionRequest = QueryValidatorCommissionRequest

class QueryValidatorCommissionResponse(google.protobuf.message.Message):
    """QueryValidatorCommissionResponse is the response type for the
    Query/ValidatorCommission RPC method
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    COMMISSION_FIELD_NUMBER: builtins.int
    @property
    def commission(self) -> cosmos.distribution.v1beta1.distribution_pb2.ValidatorAccumulatedCommission:
        """commission defines the commision the validator received."""
        pass
    def __init__(self,
        *,
        commission: typing.Optional[cosmos.distribution.v1beta1.distribution_pb2.ValidatorAccumulatedCommission] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["commission",b"commission"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["commission",b"commission"]) -> None: ...
global___QueryValidatorCommissionResponse = QueryValidatorCommissionResponse

class QueryValidatorSlashesRequest(google.protobuf.message.Message):
    """QueryValidatorSlashesRequest is the request type for the
    Query/ValidatorSlashes RPC method
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    VALIDATOR_ADDRESS_FIELD_NUMBER: builtins.int
    STARTING_HEIGHT_FIELD_NUMBER: builtins.int
    ENDING_HEIGHT_FIELD_NUMBER: builtins.int
    PAGINATION_FIELD_NUMBER: builtins.int
    validator_address: typing.Text
    """validator_address defines the validator address to query for."""

    starting_height: builtins.int
    """starting_height defines the optional starting height to query the slashes."""

    ending_height: builtins.int
    """starting_height defines the optional ending height to query the slashes."""

    @property
    def pagination(self) -> cosmos.base.query.v1beta1.pagination_pb2.PageRequest:
        """pagination defines an optional pagination for the request."""
        pass
    def __init__(self,
        *,
        validator_address: typing.Text = ...,
        starting_height: builtins.int = ...,
        ending_height: builtins.int = ...,
        pagination: typing.Optional[cosmos.base.query.v1beta1.pagination_pb2.PageRequest] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["pagination",b"pagination"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["ending_height",b"ending_height","pagination",b"pagination","starting_height",b"starting_height","validator_address",b"validator_address"]) -> None: ...
global___QueryValidatorSlashesRequest = QueryValidatorSlashesRequest

class QueryValidatorSlashesResponse(google.protobuf.message.Message):
    """QueryValidatorSlashesResponse is the response type for the
    Query/ValidatorSlashes RPC method.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    SLASHES_FIELD_NUMBER: builtins.int
    PAGINATION_FIELD_NUMBER: builtins.int
    @property
    def slashes(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.distribution.v1beta1.distribution_pb2.ValidatorSlashEvent]:
        """slashes defines the slashes the validator received."""
        pass
    @property
    def pagination(self) -> cosmos.base.query.v1beta1.pagination_pb2.PageResponse:
        """pagination defines the pagination in the response."""
        pass
    def __init__(self,
        *,
        slashes: typing.Optional[typing.Iterable[cosmos.distribution.v1beta1.distribution_pb2.ValidatorSlashEvent]] = ...,
        pagination: typing.Optional[cosmos.base.query.v1beta1.pagination_pb2.PageResponse] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["pagination",b"pagination"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["pagination",b"pagination","slashes",b"slashes"]) -> None: ...
global___QueryValidatorSlashesResponse = QueryValidatorSlashesResponse

class QueryDelegationRewardsRequest(google.protobuf.message.Message):
    """QueryDelegationRewardsRequest is the request type for the
    Query/DelegationRewards RPC method.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    DELEGATOR_ADDRESS_FIELD_NUMBER: builtins.int
    VALIDATOR_ADDRESS_FIELD_NUMBER: builtins.int
    delegator_address: typing.Text
    """delegator_address defines the delegator address to query for."""

    validator_address: typing.Text
    """validator_address defines the validator address to query for."""

    def __init__(self,
        *,
        delegator_address: typing.Text = ...,
        validator_address: typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["delegator_address",b"delegator_address","validator_address",b"validator_address"]) -> None: ...
global___QueryDelegationRewardsRequest = QueryDelegationRewardsRequest

class QueryDelegationRewardsResponse(google.protobuf.message.Message):
    """QueryDelegationRewardsResponse is the response type for the
    Query/DelegationRewards RPC method.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    REWARDS_FIELD_NUMBER: builtins.int
    @property
    def rewards(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.base.v1beta1.coin_pb2.DecCoin]:
        """rewards defines the rewards accrued by a delegation."""
        pass
    def __init__(self,
        *,
        rewards: typing.Optional[typing.Iterable[cosmos.base.v1beta1.coin_pb2.DecCoin]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["rewards",b"rewards"]) -> None: ...
global___QueryDelegationRewardsResponse = QueryDelegationRewardsResponse

class QueryDelegationTotalRewardsRequest(google.protobuf.message.Message):
    """QueryDelegationTotalRewardsRequest is the request type for the
    Query/DelegationTotalRewards RPC method.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    DELEGATOR_ADDRESS_FIELD_NUMBER: builtins.int
    delegator_address: typing.Text
    """delegator_address defines the delegator address to query for."""

    def __init__(self,
        *,
        delegator_address: typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["delegator_address",b"delegator_address"]) -> None: ...
global___QueryDelegationTotalRewardsRequest = QueryDelegationTotalRewardsRequest

class QueryDelegationTotalRewardsResponse(google.protobuf.message.Message):
    """QueryDelegationTotalRewardsResponse is the response type for the
    Query/DelegationTotalRewards RPC method.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    REWARDS_FIELD_NUMBER: builtins.int
    TOTAL_FIELD_NUMBER: builtins.int
    @property
    def rewards(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.distribution.v1beta1.distribution_pb2.DelegationDelegatorReward]:
        """rewards defines all the rewards accrued by a delegator."""
        pass
    @property
    def total(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.base.v1beta1.coin_pb2.DecCoin]:
        """total defines the sum of all the rewards."""
        pass
    def __init__(self,
        *,
        rewards: typing.Optional[typing.Iterable[cosmos.distribution.v1beta1.distribution_pb2.DelegationDelegatorReward]] = ...,
        total: typing.Optional[typing.Iterable[cosmos.base.v1beta1.coin_pb2.DecCoin]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["rewards",b"rewards","total",b"total"]) -> None: ...
global___QueryDelegationTotalRewardsResponse = QueryDelegationTotalRewardsResponse

class QueryDelegatorValidatorsRequest(google.protobuf.message.Message):
    """QueryDelegatorValidatorsRequest is the request type for the
    Query/DelegatorValidators RPC method.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    DELEGATOR_ADDRESS_FIELD_NUMBER: builtins.int
    delegator_address: typing.Text
    """delegator_address defines the delegator address to query for."""

    def __init__(self,
        *,
        delegator_address: typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["delegator_address",b"delegator_address"]) -> None: ...
global___QueryDelegatorValidatorsRequest = QueryDelegatorValidatorsRequest

class QueryDelegatorValidatorsResponse(google.protobuf.message.Message):
    """QueryDelegatorValidatorsResponse is the response type for the
    Query/DelegatorValidators RPC method.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    VALIDATORS_FIELD_NUMBER: builtins.int
    @property
    def validators(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[typing.Text]:
        """validators defines the validators a delegator is delegating for."""
        pass
    def __init__(self,
        *,
        validators: typing.Optional[typing.Iterable[typing.Text]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["validators",b"validators"]) -> None: ...
global___QueryDelegatorValidatorsResponse = QueryDelegatorValidatorsResponse

class QueryDelegatorWithdrawAddressRequest(google.protobuf.message.Message):
    """QueryDelegatorWithdrawAddressRequest is the request type for the
    Query/DelegatorWithdrawAddress RPC method.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    DELEGATOR_ADDRESS_FIELD_NUMBER: builtins.int
    delegator_address: typing.Text
    """delegator_address defines the delegator address to query for."""

    def __init__(self,
        *,
        delegator_address: typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["delegator_address",b"delegator_address"]) -> None: ...
global___QueryDelegatorWithdrawAddressRequest = QueryDelegatorWithdrawAddressRequest

class QueryDelegatorWithdrawAddressResponse(google.protobuf.message.Message):
    """QueryDelegatorWithdrawAddressResponse is the response type for the
    Query/DelegatorWithdrawAddress RPC method.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    WITHDRAW_ADDRESS_FIELD_NUMBER: builtins.int
    withdraw_address: typing.Text
    """withdraw_address defines the delegator address to query for."""

    def __init__(self,
        *,
        withdraw_address: typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["withdraw_address",b"withdraw_address"]) -> None: ...
global___QueryDelegatorWithdrawAddressResponse = QueryDelegatorWithdrawAddressResponse

class QueryCommunityPoolRequest(google.protobuf.message.Message):
    """QueryCommunityPoolRequest is the request type for the Query/CommunityPool RPC
    method.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    def __init__(self,
        ) -> None: ...
global___QueryCommunityPoolRequest = QueryCommunityPoolRequest

class QueryCommunityPoolResponse(google.protobuf.message.Message):
    """QueryCommunityPoolResponse is the response type for the Query/CommunityPool
    RPC method.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    POOL_FIELD_NUMBER: builtins.int
    @property
    def pool(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.base.v1beta1.coin_pb2.DecCoin]:
        """pool defines community pool's coins."""
        pass
    def __init__(self,
        *,
        pool: typing.Optional[typing.Iterable[cosmos.base.v1beta1.coin_pb2.DecCoin]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["pool",b"pool"]) -> None: ...
global___QueryCommunityPoolResponse = QueryCommunityPoolResponse



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import abc
import cosmos.gov.v1beta1.tx_pb2
import grpc

class MsgStub:
    """Msg defines the bank Msg service."""
    def __init__(self, channel: grpc.Channel) -> None: ...
    SubmitProposal: grpc.UnaryUnaryMultiCallable[
        cosmos.gov.v1beta1.tx_pb2.MsgSubmitProposal,
        cosmos.gov.v1beta1.tx_pb2.MsgSubmitProposalResponse]
    """SubmitProposal defines a method to create new proposal given a content."""

    Vote: grpc.UnaryUnaryMultiCallable[
        cosmos.gov.v1beta1.tx_pb2.MsgVote,
        cosmos.gov.v1beta1.tx_pb2.MsgVoteResponse]
    """Vote defines a method to add a vote on a specific proposal."""

    VoteWeighted: grpc.UnaryUnaryMultiCallable[
        cosmos.gov.v1beta1.tx_pb2.MsgVoteWeighted,
        cosmos.gov.v1beta1.tx_pb2.MsgVoteWeightedResponse]
    """VoteWeighted defines a method to add a weighted vote on a specific proposal.

    Since: cosmos-sdk 0.43
    """

    Deposit: grpc.UnaryUnaryMultiCallable[
        cosmos.gov.v1beta1.tx_pb2.MsgDeposit,
        cosmos.gov.v1beta1.tx_pb2.MsgDepositResponse]
    """Deposit defines a method to add deposit on a specific proposal."""


class MsgServicer(metaclass=abc.ABCMeta):
    """Msg defines the bank Msg service."""
    @abc.abstractmethod
    def SubmitProposal(self,
        request: cosmos.gov.v1beta1.tx_pb2.MsgSubmitProposal,
        context: grpc.ServicerContext,
    ) -> cosmos.gov.v1beta1.tx_pb2.MsgSubmitProposalResponse:
        """SubmitProposal defines a method to create new proposal given a content."""
        pass

    @abc.abstractmethod
    def Vote(self,
        request: cosmos.gov.v1beta1.tx_pb2.MsgVote,
        context: grpc.ServicerContext,
    ) -> cosmos.gov.v1beta1.tx_pb2.MsgVoteResponse:
        """Vote defines a method to add a vote on a specific proposal."""
        pass

    @abc.abstractmethod
    def VoteWeighted(self,
        request: cosmos.gov.v1beta1.tx_pb2.MsgVoteWeighted,
        context: grpc.ServicerContext,
    ) -> cosmos.gov.v1beta1.tx_pb2.MsgVoteWeightedResponse:
        """VoteWeighted defines a method to add a weighted vote on a specific proposal.

        Since: cosmos-sdk 0.43
        """
        pass

    @abc.abstractmethod
    def Deposit(self,
        request: cosmos.gov.v1beta1.tx_pb2.MsgDeposit,
        context: grpc.ServicerContext,
    ) -> cosmos.gov.v1beta1.tx_pb2.MsgDepositResponse:
        """Deposit defines a method to add deposit on a specific proposal."""
        pass


def add_MsgServicer_to_server(servicer: MsgServicer, server: grpc.Server) -> None: ...



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class AddOracleProposal(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    TITLE_FIELD_NUMBER: builtins.int
    DESCRIPTION_FIELD_NUMBER: builtins.int
    ORACLES_FIELD_NUMBER: builtins.int
    PAIRS_FIELD_NUMBER: builtins.int
    title: typing.Text
    description: typing.Text
    @property
    def oracles(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[typing.Text]: ...
    @property
    def pairs(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[typing.Text]: ...
    def __init__(self,
        *,
        title: typing.Text = ...,
        description: typing.Text = ...,
        oracles: typing.Optional[typing.Iterable[typing.Text]] = ...,
        pairs: typing.Optional[typing.Iterable[typing.Text]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["description",b"description","oracles",b"oracles","pairs",b"pairs","title",b"title"]) -> None: ...
global___AddOracleProposal = AddOracleProposal



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import abc
import cosmos.bank.v1beta1.query_pb2
import grpc

class QueryStub:
    """Query defines the gRPC querier service."""
    def __init__(self, channel: grpc.Channel) -> None: ...
    Balance: grpc.UnaryUnaryMultiCallable[
        cosmos.bank.v1beta1.query_pb2.QueryBalanceRequest,
        cosmos.bank.v1beta1.query_pb2.QueryBalanceResponse]
    """Balance queries the balance of a single coin for a single account."""

    AllBalances: grpc.UnaryUnaryMultiCallable[
        cosmos.bank.v1beta1.query_pb2.QueryAllBalancesRequest,
        cosmos.bank.v1beta1.query_pb2.QueryAllBalancesResponse]
    """AllBalances queries the balance of all coins for a single account."""

    SpendableBalances: grpc.UnaryUnaryMultiCallable[
        cosmos.bank.v1beta1.query_pb2.QuerySpendableBalancesRequest,
        cosmos.bank.v1beta1.query_pb2.QuerySpendableBalancesResponse]
    """SpendableBalances queries the spenable balance of all coins for a single
    account.
    """

    TotalSupply: grpc.UnaryUnaryMultiCallable[
        cosmos.bank.v1beta1.query_pb2.QueryTotalSupplyRequest,
        cosmos.bank.v1beta1.query_pb2.QueryTotalSupplyResponse]
    """TotalSupply queries the total supply of all coins."""

    SupplyOf: grpc.UnaryUnaryMultiCallable[
        cosmos.bank.v1beta1.query_pb2.QuerySupplyOfRequest,
        cosmos.bank.v1beta1.query_pb2.QuerySupplyOfResponse]
    """SupplyOf queries the supply of a single coin."""

    Params: grpc.UnaryUnaryMultiCallable[
        cosmos.bank.v1beta1.query_pb2.QueryParamsRequest,
        cosmos.bank.v1beta1.query_pb2.QueryParamsResponse]
    """Params queries the parameters of x/bank module."""

    DenomMetadata: grpc.UnaryUnaryMultiCallable[
        cosmos.bank.v1beta1.query_pb2.QueryDenomMetadataRequest,
        cosmos.bank.v1beta1.query_pb2.QueryDenomMetadataResponse]
    """DenomsMetadata queries the client metadata of a given coin denomination."""

    DenomsMetadata: grpc.UnaryUnaryMultiCallable[
        cosmos.bank.v1beta1.query_pb2.QueryDenomsMetadataRequest,
        cosmos.bank.v1beta1.query_pb2.QueryDenomsMetadataResponse]
    """DenomsMetadata queries the client metadata for all registered coin denominations."""


class QueryServicer(metaclass=abc.ABCMeta):
    """Query defines the gRPC querier service."""
    @abc.abstractmethod
    def Balance(self,
        request: cosmos.bank.v1beta1.query_pb2.QueryBalanceRequest,
        context: grpc.ServicerContext,
    ) -> cosmos.bank.v1beta1.query_pb2.QueryBalanceResponse:
        """Balance queries the balance of a single coin for a single account."""
        pass

    @abc.abstractmethod
    def AllBalances(self,
        request: cosmos.bank.v1beta1.query_pb2.QueryAllBalancesRequest,
        context: grpc.ServicerContext,
    ) -> cosmos.bank.v1beta1.query_pb2.QueryAllBalancesResponse:
        """AllBalances queries the balance of all coins for a single account."""
        pass

    @abc.abstractmethod
    def SpendableBalances(self,
        request: cosmos.bank.v1beta1.query_pb2.QuerySpendableBalancesRequest,
        context: grpc.ServicerContext,
    ) -> cosmos.bank.v1beta1.query_pb2.QuerySpendableBalancesResponse:
        """SpendableBalances queries the spenable balance of all coins for a single
        account.
        """
        pass

    @abc.abstractmethod
    def TotalSupply(self,
        request: cosmos.bank.v1beta1.query_pb2.QueryTotalSupplyRequest,
        context: grpc.ServicerContext,
    ) -> cosmos.bank.v1beta1.query_pb2.QueryTotalSupplyResponse:
        """TotalSupply queries the total supply of all coins."""
        pass

    @abc.abstractmethod
    def SupplyOf(self,
        request: cosmos.bank.v1beta1.query_pb2.QuerySupplyOfRequest,
        context: grpc.ServicerContext,
    ) -> cosmos.bank.v1beta1.query_pb2.QuerySupplyOfResponse:
        """SupplyOf queries the supply of a single coin."""
        pass

    @abc.abstractmethod
    def Params(self,
        request: cosmos.bank.v1beta1.query_pb2.QueryParamsRequest,
        context: grpc.ServicerContext,
    ) -> cosmos.bank.v1beta1.query_pb2.QueryParamsResponse:
        """Params queries the parameters of x/bank module."""
        pass

    @abc.abstractmethod
    def DenomMetadata(self,
        request: cosmos.bank.v1beta1.query_pb2.QueryDenomMetadataRequest,
        context: grpc.ServicerContext,
    ) -> cosmos.bank.v1beta1.query_pb2.QueryDenomMetadataResponse:
        """DenomsMetadata queries the client metadata of a given coin denomination."""
        pass

    @abc.abstractmethod
    def DenomsMetadata(self,
        request: cosmos.bank.v1beta1.query_pb2.QueryDenomsMetadataRequest,
        context: grpc.ServicerContext,
    ) -> cosmos.bank.v1beta1.query_pb2.QueryDenomsMetadataResponse:
        """DenomsMetadata queries the client metadata for all registered coin denominations."""
        pass


def add_QueryServicer_to_server(servicer: QueryServicer, server: grpc.Server) -> None: ...



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.any_pb2
import google.protobuf.descriptor
import google.protobuf.message
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class MsgSubmitEvidence(google.protobuf.message.Message):
    """MsgSubmitEvidence represents a message that supports submitting arbitrary
    Evidence of misbehavior such as equivocation or counterfactual signing.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    SUBMITTER_FIELD_NUMBER: builtins.int
    EVIDENCE_FIELD_NUMBER: builtins.int
    submitter: typing.Text
    @property
    def evidence(self) -> google.protobuf.any_pb2.Any: ...
    def __init__(self,
        *,
        submitter: typing.Text = ...,
        evidence: typing.Optional[google.protobuf.any_pb2.Any] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["evidence",b"evidence"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["evidence",b"evidence","submitter",b"submitter"]) -> None: ...
global___MsgSubmitEvidence = MsgSubmitEvidence

class MsgSubmitEvidenceResponse(google.protobuf.message.Message):
    """MsgSubmitEvidenceResponse defines the Msg/SubmitEvidence response type."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    HASH_FIELD_NUMBER: builtins.int
    hash: builtins.bytes
    """hash defines the hash of the evidence."""

    def __init__(self,
        *,
        hash: builtins.bytes = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["hash",b"hash"]) -> None: ...
global___MsgSubmitEvidenceResponse = MsgSubmitEvidenceResponse



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import abc
import cosmos.slashing.v1beta1.tx_pb2
import grpc

class MsgStub:
    """Msg defines the slashing Msg service."""
    def __init__(self, channel: grpc.Channel) -> None: ...
    Unjail: grpc.UnaryUnaryMultiCallable[
        cosmos.slashing.v1beta1.tx_pb2.MsgUnjail,
        cosmos.slashing.v1beta1.tx_pb2.MsgUnjailResponse]
    """Unjail defines a method for unjailing a jailed validator, thus returning
    them into the bonded validator set, so they can begin receiving provisions
    and rewards again.
    """


class MsgServicer(metaclass=abc.ABCMeta):
    """Msg defines the slashing Msg service."""
    @abc.abstractmethod
    def Unjail(self,
        request: cosmos.slashing.v1beta1.tx_pb2.MsgUnjail,
        context: grpc.ServicerContext,
    ) -> cosmos.slashing.v1beta1.tx_pb2.MsgUnjailResponse:
        """Unjail defines a method for unjailing a jailed validator, thus returning
        them into the bonded validator set, so they can begin receiving provisions
        and rewards again.
        """
        pass


def add_MsgServicer_to_server(servicer: MsgServicer, server: grpc.Server) -> None: ...



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import oracle.v1beta1.oracle_pb2
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class QueryExchangeRateRequest(google.protobuf.message.Message):
    """QueryExchangeRateRequest is the request type for the Query/ExchangeRate RPC method."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    PAIR_FIELD_NUMBER: builtins.int
    pair: typing.Text
    """pair defines the pair to query for."""

    def __init__(self,
        *,
        pair: typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["pair",b"pair"]) -> None: ...
global___QueryExchangeRateRequest = QueryExchangeRateRequest

class QueryExchangeRateResponse(google.protobuf.message.Message):
    """QueryExchangeRateResponse is response type for the
    Query/ExchangeRate RPC method.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    EXCHANGE_RATE_FIELD_NUMBER: builtins.int
    exchange_rate: typing.Text
    """exchange_rate defines the exchange rate of assets voted by validators"""

    def __init__(self,
        *,
        exchange_rate: typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["exchange_rate",b"exchange_rate"]) -> None: ...
global___QueryExchangeRateResponse = QueryExchangeRateResponse

class QueryExchangeRatesRequest(google.protobuf.message.Message):
    """QueryExchangeRatesRequest is the request type for the Query/ExchangeRates RPC method."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    def __init__(self,
        ) -> None: ...
global___QueryExchangeRatesRequest = QueryExchangeRatesRequest

class QueryExchangeRatesResponse(google.protobuf.message.Message):
    """QueryExchangeRatesResponse is response type for the
    Query/ExchangeRates RPC method.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    EXCHANGE_RATES_FIELD_NUMBER: builtins.int
    @property
    def exchange_rates(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[oracle.v1beta1.oracle_pb2.ExchangeRateTuple]:
        """exchange_rates defines a list of the exchange rate for all whitelisted pairs."""
        pass
    def __init__(self,
        *,
        exchange_rates: typing.Optional[typing.Iterable[oracle.v1beta1.oracle_pb2.ExchangeRateTuple]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["exchange_rates",b"exchange_rates"]) -> None: ...
global___QueryExchangeRatesResponse = QueryExchangeRatesResponse

class QueryTobinTaxRequest(google.protobuf.message.Message):
    """QueryTobinTaxRequest is the request type for the Query/TobinTax RPC method."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    PAIR_FIELD_NUMBER: builtins.int
    pair: typing.Text
    """pair defines the asset to query for."""

    def __init__(self,
        *,
        pair: typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["pair",b"pair"]) -> None: ...
global___QueryTobinTaxRequest = QueryTobinTaxRequest

class QueryTobinTaxResponse(google.protobuf.message.Message):
    """QueryTobinTaxResponse is response type for the
    Query/TobinTax RPC method.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    TOBIN_TAX_FIELD_NUMBER: builtins.int
    tobin_tax: typing.Text
    """tobin_taxe defines the tobin tax of a pair"""

    def __init__(self,
        *,
        tobin_tax: typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["tobin_tax",b"tobin_tax"]) -> None: ...
global___QueryTobinTaxResponse = QueryTobinTaxResponse

class QueryTobinTaxesRequest(google.protobuf.message.Message):
    """QueryTobinTaxesRequest is the request type for the Query/TobinTaxes RPC method."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    def __init__(self,
        ) -> None: ...
global___QueryTobinTaxesRequest = QueryTobinTaxesRequest

class QueryTobinTaxesResponse(google.protobuf.message.Message):
    """QueryTobinTaxesResponse is response type for the
    Query/TobinTaxes RPC method.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    TOBIN_TAXES_FIELD_NUMBER: builtins.int
    @property
    def tobin_taxes(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[oracle.v1beta1.oracle_pb2.Pair]:
        """tobin_taxes defines a list of the tobin tax of all whitelisted pairs"""
        pass
    def __init__(self,
        *,
        tobin_taxes: typing.Optional[typing.Iterable[oracle.v1beta1.oracle_pb2.Pair]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["tobin_taxes",b"tobin_taxes"]) -> None: ...
global___QueryTobinTaxesResponse = QueryTobinTaxesResponse

class QueryActivesRequest(google.protobuf.message.Message):
    """QueryActivesRequest is the request type for the Query/Actives RPC method."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    def __init__(self,
        ) -> None: ...
global___QueryActivesRequest = QueryActivesRequest

class QueryActivesResponse(google.protobuf.message.Message):
    """QueryActivesResponse is response type for the
    Query/Actives RPC method.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    ACTIVES_FIELD_NUMBER: builtins.int
    @property
    def actives(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[typing.Text]:
        """actives defines a list of the pair which oracle prices agreed upon."""
        pass
    def __init__(self,
        *,
        actives: typing.Optional[typing.Iterable[typing.Text]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["actives",b"actives"]) -> None: ...
global___QueryActivesResponse = QueryActivesResponse

class QueryVoteTargetsRequest(google.protobuf.message.Message):
    """QueryVoteTargetsRequest is the request type for the Query/VoteTargets RPC method."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    def __init__(self,
        ) -> None: ...
global___QueryVoteTargetsRequest = QueryVoteTargetsRequest

class QueryVoteTargetsResponse(google.protobuf.message.Message):
    """QueryVoteTargetsResponse is response type for the
    Query/VoteTargets RPC method.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    VOTE_TARGETS_FIELD_NUMBER: builtins.int
    @property
    def vote_targets(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[typing.Text]:
        """vote_targets defines a list of the pairs in which everyone
        should vote in the current vote period.
        """
        pass
    def __init__(self,
        *,
        vote_targets: typing.Optional[typing.Iterable[typing.Text]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["vote_targets",b"vote_targets"]) -> None: ...
global___QueryVoteTargetsResponse = QueryVoteTargetsResponse

class QueryFeederDelegationRequest(google.protobuf.message.Message):
    """QueryFeederDelegationRequest is the request type for the Query/FeederDelegation RPC method."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    VALIDATOR_ADDR_FIELD_NUMBER: builtins.int
    validator_addr: typing.Text
    """validator defines the validator address to query for."""

    def __init__(self,
        *,
        validator_addr: typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["validator_addr",b"validator_addr"]) -> None: ...
global___QueryFeederDelegationRequest = QueryFeederDelegationRequest

class QueryFeederDelegationResponse(google.protobuf.message.Message):
    """QueryFeederDelegationResponse is response type for the
    Query/FeederDelegation RPC method.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    FEEDER_ADDR_FIELD_NUMBER: builtins.int
    feeder_addr: typing.Text
    """feeder_addr defines the feeder delegation of a validator"""

    def __init__(self,
        *,
        feeder_addr: typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["feeder_addr",b"feeder_addr"]) -> None: ...
global___QueryFeederDelegationResponse = QueryFeederDelegationResponse

class QueryMissCounterRequest(google.protobuf.message.Message):
    """QueryMissCounterRequest is the request type for the Query/MissCounter RPC method."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    VALIDATOR_ADDR_FIELD_NUMBER: builtins.int
    validator_addr: typing.Text
    """validator defines the validator address to query for."""

    def __init__(self,
        *,
        validator_addr: typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["validator_addr",b"validator_addr"]) -> None: ...
global___QueryMissCounterRequest = QueryMissCounterRequest

class QueryMissCounterResponse(google.protobuf.message.Message):
    """QueryMissCounterResponse is response type for the
    Query/MissCounter RPC method.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    MISS_COUNTER_FIELD_NUMBER: builtins.int
    miss_counter: builtins.int
    """miss_counter defines the oracle miss counter of a validator"""

    def __init__(self,
        *,
        miss_counter: builtins.int = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["miss_counter",b"miss_counter"]) -> None: ...
global___QueryMissCounterResponse = QueryMissCounterResponse

class QueryAggregatePrevoteRequest(google.protobuf.message.Message):
    """QueryAggregatePrevoteRequest is the request type for the Query/AggregatePrevote RPC method."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    VALIDATOR_ADDR_FIELD_NUMBER: builtins.int
    validator_addr: typing.Text
    """validator defines the validator address to query for."""

    def __init__(self,
        *,
        validator_addr: typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["validator_addr",b"validator_addr"]) -> None: ...
global___QueryAggregatePrevoteRequest = QueryAggregatePrevoteRequest

class QueryAggregatePrevoteResponse(google.protobuf.message.Message):
    """QueryAggregatePrevoteResponse is response type for the
    Query/AggregatePrevote RPC method.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    AGGREGATE_PREVOTE_FIELD_NUMBER: builtins.int
    @property
    def aggregate_prevote(self) -> oracle.v1beta1.oracle_pb2.AggregateExchangeRatePrevote:
        """aggregate_prevote defines oracle aggregate prevote submitted by a validator in the current vote period"""
        pass
    def __init__(self,
        *,
        aggregate_prevote: typing.Optional[oracle.v1beta1.oracle_pb2.AggregateExchangeRatePrevote] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["aggregate_prevote",b"aggregate_prevote"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["aggregate_prevote",b"aggregate_prevote"]) -> None: ...
global___QueryAggregatePrevoteResponse = QueryAggregatePrevoteResponse

class QueryAggregatePrevotesRequest(google.protobuf.message.Message):
    """QueryAggregatePrevotesRequest is the request type for the Query/AggregatePrevotes RPC method."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    def __init__(self,
        ) -> None: ...
global___QueryAggregatePrevotesRequest = QueryAggregatePrevotesRequest

class QueryAggregatePrevotesResponse(google.protobuf.message.Message):
    """QueryAggregatePrevotesResponse is response type for the
    Query/AggregatePrevotes RPC method.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    AGGREGATE_PREVOTES_FIELD_NUMBER: builtins.int
    @property
    def aggregate_prevotes(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[oracle.v1beta1.oracle_pb2.AggregateExchangeRatePrevote]:
        """aggregate_prevotes defines all oracle aggregate prevotes submitted in the current vote period"""
        pass
    def __init__(self,
        *,
        aggregate_prevotes: typing.Optional[typing.Iterable[oracle.v1beta1.oracle_pb2.AggregateExchangeRatePrevote]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["aggregate_prevotes",b"aggregate_prevotes"]) -> None: ...
global___QueryAggregatePrevotesResponse = QueryAggregatePrevotesResponse

class QueryAggregateVoteRequest(google.protobuf.message.Message):
    """QueryAggregateVoteRequest is the request type for the Query/AggregateVote RPC method."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    VALIDATOR_ADDR_FIELD_NUMBER: builtins.int
    validator_addr: typing.Text
    """validator defines the validator address to query for."""

    def __init__(self,
        *,
        validator_addr: typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["validator_addr",b"validator_addr"]) -> None: ...
global___QueryAggregateVoteRequest = QueryAggregateVoteRequest

class QueryAggregateVoteResponse(google.protobuf.message.Message):
    """QueryAggregateVoteResponse is response type for the
    Query/AggregateVote RPC method.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    AGGREGATE_VOTE_FIELD_NUMBER: builtins.int
    @property
    def aggregate_vote(self) -> oracle.v1beta1.oracle_pb2.AggregateExchangeRateVote:
        """aggregate_vote defines oracle aggregate vote submitted by a validator in the current vote period"""
        pass
    def __init__(self,
        *,
        aggregate_vote: typing.Optional[oracle.v1beta1.oracle_pb2.AggregateExchangeRateVote] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["aggregate_vote",b"aggregate_vote"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["aggregate_vote",b"aggregate_vote"]) -> None: ...
global___QueryAggregateVoteResponse = QueryAggregateVoteResponse

class QueryAggregateVotesRequest(google.protobuf.message.Message):
    """QueryAggregateVotesRequest is the request type for the Query/AggregateVotes RPC method."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    def __init__(self,
        ) -> None: ...
global___QueryAggregateVotesRequest = QueryAggregateVotesRequest

class QueryAggregateVotesResponse(google.protobuf.message.Message):
    """QueryAggregateVotesResponse is response type for the
    Query/AggregateVotes RPC method.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    AGGREGATE_VOTES_FIELD_NUMBER: builtins.int
    @property
    def aggregate_votes(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[oracle.v1beta1.oracle_pb2.AggregateExchangeRateVote]:
        """aggregate_votes defines all oracle aggregate votes submitted in the current vote period"""
        pass
    def __init__(self,
        *,
        aggregate_votes: typing.Optional[typing.Iterable[oracle.v1beta1.oracle_pb2.AggregateExchangeRateVote]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["aggregate_votes",b"aggregate_votes"]) -> None: ...
global___QueryAggregateVotesResponse = QueryAggregateVotesResponse

class QueryParamsRequest(google.protobuf.message.Message):
    """QueryParamsRequest is the request type for the Query/Params RPC method."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    def __init__(self,
        ) -> None: ...
global___QueryParamsRequest = QueryParamsRequest

class QueryParamsResponse(google.protobuf.message.Message):
    """QueryParamsResponse is the response type for the Query/Params RPC method."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    PARAMS_FIELD_NUMBER: builtins.int
    @property
    def params(self) -> oracle.v1beta1.oracle_pb2.Params:
        """params defines the parameters of the module."""
        pass
    def __init__(self,
        *,
        params: typing.Optional[oracle.v1beta1.oracle_pb2.Params] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["params",b"params"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["params",b"params"]) -> None: ...
global___QueryParamsResponse = QueryParamsResponse



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: pricefeed/snapshot.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from gogoproto import gogo_pb2 as gogoproto_dot_gogo__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from google.api import annotations_pb2 as google_dot_api_dot_annotations__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x18pricefeed/snapshot.proto\x12\x13nibiru.pricefeed.v1\x1a\x14gogoproto/gogo.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x1cgoogle/api/annotations.proto\"u\n\rPriceSnapshot\x12\x0f\n\x07pair_id\x18\x01 \x01(\t\x12=\n\x05price\x18\x02 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x12\x14\n\x0ctimestamp_ms\x18\x03 \x01(\x03\x42\x39Z/github.com/NibiruChain/nibiru/x/pricefeed/types\xa8\xe2\x1e\x01\xe0\xe1\x1e\x01\x62\x06proto3')



_PRICESNAPSHOT = DESCRIPTOR.message_types_by_name['PriceSnapshot']
PriceSnapshot = _reflection.GeneratedProtocolMessageType('PriceSnapshot', (_message.Message,), {
  'DESCRIPTOR' : _PRICESNAPSHOT,
  '__module__' : 'pricefeed.snapshot_pb2'
  # @@protoc_insertion_point(class_scope:nibiru.pricefeed.v1.PriceSnapshot)
  })
_sym_db.RegisterMessage(PriceSnapshot)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z/github.com/NibiruChain/nibiru/x/pricefeed/types\250\342\036\001\340\341\036\001'
  _PRICESNAPSHOT.fields_by_name['price']._options = None
  _PRICESNAPSHOT.fields_by_name['price']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _PRICESNAPSHOT._serialized_start=134
  _PRICESNAPSHOT._serialized_end=251
# @@protoc_insertion_point(module_scope)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import abc
import grpc
import incentivization.v1.incentivization_pb2

class MsgStub:
    def __init__(self, channel: grpc.Channel) -> None: ...
    CreateIncentivizationProgram: grpc.UnaryUnaryMultiCallable[
        incentivization.v1.incentivization_pb2.MsgCreateIncentivizationProgram,
        incentivization.v1.incentivization_pb2.MsgCreateIncentivizationProgramResponse]
    """CreateIncentivizationProgram allows an entity to create an incentivization program for a liquidity pool."""

    FundIncentivizationProgram: grpc.UnaryUnaryMultiCallable[
        incentivization.v1.incentivization_pb2.MsgFundIncentivizationProgram,
        incentivization.v1.incentivization_pb2.MsgFundIncentivizationProgramResponse]
    """FundIncentivizationProgram allows an entity to fund an already existing incentivization program with more coins."""


class MsgServicer(metaclass=abc.ABCMeta):
    @abc.abstractmethod
    def CreateIncentivizationProgram(self,
        request: incentivization.v1.incentivization_pb2.MsgCreateIncentivizationProgram,
        context: grpc.ServicerContext,
    ) -> incentivization.v1.incentivization_pb2.MsgCreateIncentivizationProgramResponse:
        """CreateIncentivizationProgram allows an entity to create an incentivization program for a liquidity pool."""
        pass

    @abc.abstractmethod
    def FundIncentivizationProgram(self,
        request: incentivization.v1.incentivization_pb2.MsgFundIncentivizationProgram,
        context: grpc.ServicerContext,
    ) -> incentivization.v1.incentivization_pb2.MsgFundIncentivizationProgramResponse:
        """FundIncentivizationProgram allows an entity to fund an already existing incentivization program with more coins."""
        pass


def add_MsgServicer_to_server(servicer: MsgServicer, server: grpc.Server) -> None: ...

class QueryStub:
    def __init__(self, channel: grpc.Channel) -> None: ...
    IncentivizationProgram: grpc.UnaryUnaryMultiCallable[
        incentivization.v1.incentivization_pb2.QueryIncentivizationProgramRequest,
        incentivization.v1.incentivization_pb2.QueryIncentivizationProgramResponse]

    IncentivizationPrograms: grpc.UnaryUnaryMultiCallable[
        incentivization.v1.incentivization_pb2.QueryIncentivizationProgramsRequest,
        incentivization.v1.incentivization_pb2.QueryIncentivizationProgramsResponse]


class QueryServicer(metaclass=abc.ABCMeta):
    @abc.abstractmethod
    def IncentivizationProgram(self,
        request: incentivization.v1.incentivization_pb2.QueryIncentivizationProgramRequest,
        context: grpc.ServicerContext,
    ) -> incentivization.v1.incentivization_pb2.QueryIncentivizationProgramResponse: ...

    @abc.abstractmethod
    def IncentivizationPrograms(self,
        request: incentivization.v1.incentivization_pb2.QueryIncentivizationProgramsRequest,
        context: grpc.ServicerContext,
    ) -> incentivization.v1.incentivization_pb2.QueryIncentivizationProgramsResponse: ...


def add_QueryServicer_to_server(servicer: QueryServicer, server: grpc.Server) -> None: ...



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: oracle/v1beta1/tx.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from gogoproto import gogo_pb2 as gogoproto_dot_gogo__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x17oracle/v1beta1/tx.proto\x12\x15nibiru.oracle.v1beta1\x1a\x14gogoproto/gogo.proto\"\x96\x01\n\x1fMsgAggregateExchangeRatePrevote\x12\x1d\n\x04hash\x18\x01 \x01(\tB\x0f\xf2\xde\x1f\x0byaml:\"hash\"\x12!\n\x06\x66\x65\x65\x64\x65r\x18\x02 \x01(\tB\x11\xf2\xde\x1f\ryaml:\"feeder\"\x12\'\n\tvalidator\x18\x03 \x01(\tB\x14\xf2\xde\x1f\x10yaml:\"validator\":\x08\xe8\xa0\x1f\x00\x88\xa0\x1f\x00\")\n\'MsgAggregateExchangeRatePrevoteResponse\"\xc6\x01\n\x1cMsgAggregateExchangeRateVote\x12\x1d\n\x04salt\x18\x01 \x01(\tB\x0f\xf2\xde\x1f\x0byaml:\"salt\"\x12\x31\n\x0e\x65xchange_rates\x18\x02 \x01(\tB\x19\xf2\xde\x1f\x15yaml:\"exchange_rates\"\x12!\n\x06\x66\x65\x65\x64\x65r\x18\x03 \x01(\tB\x11\xf2\xde\x1f\ryaml:\"feeder\"\x12\'\n\tvalidator\x18\x04 \x01(\tB\x14\xf2\xde\x1f\x10yaml:\"validator\":\x08\xe8\xa0\x1f\x00\x88\xa0\x1f\x00\"&\n$MsgAggregateExchangeRateVoteResponse\"p\n\x16MsgDelegateFeedConsent\x12%\n\x08operator\x18\x01 \x01(\tB\x13\xf2\xde\x1f\x0fyaml:\"operator\"\x12%\n\x08\x64\x65legate\x18\x02 \x01(\tB\x13\xf2\xde\x1f\x0fyaml:\"delegate\":\x08\xe8\xa0\x1f\x00\x88\xa0\x1f\x00\" \n\x1eMsgDelegateFeedConsentResponse2\xab\x03\n\x03Msg\x12\x96\x01\n\x1c\x41ggregateExchangeRatePrevote\x12\x36.nibiru.oracle.v1beta1.MsgAggregateExchangeRatePrevote\x1a>.nibiru.oracle.v1beta1.MsgAggregateExchangeRatePrevoteResponse\x12\x8d\x01\n\x19\x41ggregateExchangeRateVote\x12\x33.nibiru.oracle.v1beta1.MsgAggregateExchangeRateVote\x1a;.nibiru.oracle.v1beta1.MsgAggregateExchangeRateVoteResponse\x12{\n\x13\x44\x65legateFeedConsent\x12-.nibiru.oracle.v1beta1.MsgDelegateFeedConsent\x1a\x35.nibiru.oracle.v1beta1.MsgDelegateFeedConsentResponseB.Z,github.com/NibiruChain/nibiru/x/oracle/typesb\x06proto3')



_MSGAGGREGATEEXCHANGERATEPREVOTE = DESCRIPTOR.message_types_by_name['MsgAggregateExchangeRatePrevote']
_MSGAGGREGATEEXCHANGERATEPREVOTERESPONSE = DESCRIPTOR.message_types_by_name['MsgAggregateExchangeRatePrevoteResponse']
_MSGAGGREGATEEXCHANGERATEVOTE = DESCRIPTOR.message_types_by_name['MsgAggregateExchangeRateVote']
_MSGAGGREGATEEXCHANGERATEVOTERESPONSE = DESCRIPTOR.message_types_by_name['MsgAggregateExchangeRateVoteResponse']
_MSGDELEGATEFEEDCONSENT = DESCRIPTOR.message_types_by_name['MsgDelegateFeedConsent']
_MSGDELEGATEFEEDCONSENTRESPONSE = DESCRIPTOR.message_types_by_name['MsgDelegateFeedConsentResponse']
MsgAggregateExchangeRatePrevote = _reflection.GeneratedProtocolMessageType('MsgAggregateExchangeRatePrevote', (_message.Message,), {
  'DESCRIPTOR' : _MSGAGGREGATEEXCHANGERATEPREVOTE,
  '__module__' : 'oracle.v1beta1.tx_pb2'
  # @@protoc_insertion_point(class_scope:nibiru.oracle.v1beta1.MsgAggregateExchangeRatePrevote)
  })
_sym_db.RegisterMessage(MsgAggregateExchangeRatePrevote)

MsgAggregateExchangeRatePrevoteResponse = _reflection.GeneratedProtocolMessageType('MsgAggregateExchangeRatePrevoteResponse', (_message.Message,), {
  'DESCRIPTOR' : _MSGAGGREGATEEXCHANGERATEPREVOTERESPONSE,
  '__module__' : 'oracle.v1beta1.tx_pb2'
  # @@protoc_insertion_point(class_scope:nibiru.oracle.v1beta1.MsgAggregateExchangeRatePrevoteResponse)
  })
_sym_db.RegisterMessage(MsgAggregateExchangeRatePrevoteResponse)

MsgAggregateExchangeRateVote = _reflection.GeneratedProtocolMessageType('MsgAggregateExchangeRateVote', (_message.Message,), {
  'DESCRIPTOR' : _MSGAGGREGATEEXCHANGERATEVOTE,
  '__module__' : 'oracle.v1beta1.tx_pb2'
  # @@protoc_insertion_point(class_scope:nibiru.oracle.v1beta1.MsgAggregateExchangeRateVote)
  })
_sym_db.RegisterMessage(MsgAggregateExchangeRateVote)

MsgAggregateExchangeRateVoteResponse = _reflection.GeneratedProtocolMessageType('MsgAggregateExchangeRateVoteResponse', (_message.Message,), {
  'DESCRIPTOR' : _MSGAGGREGATEEXCHANGERATEVOTERESPONSE,
  '__module__' : 'oracle.v1beta1.tx_pb2'
  # @@protoc_insertion_point(class_scope:nibiru.oracle.v1beta1.MsgAggregateExchangeRateVoteResponse)
  })
_sym_db.RegisterMessage(MsgAggregateExchangeRateVoteResponse)

MsgDelegateFeedConsent = _reflection.GeneratedProtocolMessageType('MsgDelegateFeedConsent', (_message.Message,), {
  'DESCRIPTOR' : _MSGDELEGATEFEEDCONSENT,
  '__module__' : 'oracle.v1beta1.tx_pb2'
  # @@protoc_insertion_point(class_scope:nibiru.oracle.v1beta1.MsgDelegateFeedConsent)
  })
_sym_db.RegisterMessage(MsgDelegateFeedConsent)

MsgDelegateFeedConsentResponse = _reflection.GeneratedProtocolMessageType('MsgDelegateFeedConsentResponse', (_message.Message,), {
  'DESCRIPTOR' : _MSGDELEGATEFEEDCONSENTRESPONSE,
  '__module__' : 'oracle.v1beta1.tx_pb2'
  # @@protoc_insertion_point(class_scope:nibiru.oracle.v1beta1.MsgDelegateFeedConsentResponse)
  })
_sym_db.RegisterMessage(MsgDelegateFeedConsentResponse)

_MSG = DESCRIPTOR.services_by_name['Msg']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z,github.com/NibiruChain/nibiru/x/oracle/types'
  _MSGAGGREGATEEXCHANGERATEPREVOTE.fields_by_name['hash']._options = None
  _MSGAGGREGATEEXCHANGERATEPREVOTE.fields_by_name['hash']._serialized_options = b'\362\336\037\013yaml:\"hash\"'
  _MSGAGGREGATEEXCHANGERATEPREVOTE.fields_by_name['feeder']._options = None
  _MSGAGGREGATEEXCHANGERATEPREVOTE.fields_by_name['feeder']._serialized_options = b'\362\336\037\ryaml:\"feeder\"'
  _MSGAGGREGATEEXCHANGERATEPREVOTE.fields_by_name['validator']._options = None
  _MSGAGGREGATEEXCHANGERATEPREVOTE.fields_by_name['validator']._serialized_options = b'\362\336\037\020yaml:\"validator\"'
  _MSGAGGREGATEEXCHANGERATEPREVOTE._options = None
  _MSGAGGREGATEEXCHANGERATEPREVOTE._serialized_options = b'\350\240\037\000\210\240\037\000'
  _MSGAGGREGATEEXCHANGERATEVOTE.fields_by_name['salt']._options = None
  _MSGAGGREGATEEXCHANGERATEVOTE.fields_by_name['salt']._serialized_options = b'\362\336\037\013yaml:\"salt\"'
  _MSGAGGREGATEEXCHANGERATEVOTE.fields_by_name['exchange_rates']._options = None
  _MSGAGGREGATEEXCHANGERATEVOTE.fields_by_name['exchange_rates']._serialized_options = b'\362\336\037\025yaml:\"exchange_rates\"'
  _MSGAGGREGATEEXCHANGERATEVOTE.fields_by_name['feeder']._options = None
  _MSGAGGREGATEEXCHANGERATEVOTE.fields_by_name['feeder']._serialized_options = b'\362\336\037\ryaml:\"feeder\"'
  _MSGAGGREGATEEXCHANGERATEVOTE.fields_by_name['validator']._options = None
  _MSGAGGREGATEEXCHANGERATEVOTE.fields_by_name['validator']._serialized_options = b'\362\336\037\020yaml:\"validator\"'
  _MSGAGGREGATEEXCHANGERATEVOTE._options = None
  _MSGAGGREGATEEXCHANGERATEVOTE._serialized_options = b'\350\240\037\000\210\240\037\000'
  _MSGDELEGATEFEEDCONSENT.fields_by_name['operator']._options = None
  _MSGDELEGATEFEEDCONSENT.fields_by_name['operator']._serialized_options = b'\362\336\037\017yaml:\"operator\"'
  _MSGDELEGATEFEEDCONSENT.fields_by_name['delegate']._options = None
  _MSGDELEGATEFEEDCONSENT.fields_by_name['delegate']._serialized_options = b'\362\336\037\017yaml:\"delegate\"'
  _MSGDELEGATEFEEDCONSENT._options = None
  _MSGDELEGATEFEEDCONSENT._serialized_options = b'\350\240\037\000\210\240\037\000'
  _MSGAGGREGATEEXCHANGERATEPREVOTE._serialized_start=73
  _MSGAGGREGATEEXCHANGERATEPREVOTE._serialized_end=223
  _MSGAGGREGATEEXCHANGERATEPREVOTERESPONSE._serialized_start=225
  _MSGAGGREGATEEXCHANGERATEPREVOTERESPONSE._serialized_end=266
  _MSGAGGREGATEEXCHANGERATEVOTE._serialized_start=269
  _MSGAGGREGATEEXCHANGERATEVOTE._serialized_end=467
  _MSGAGGREGATEEXCHANGERATEVOTERESPONSE._serialized_start=469
  _MSGAGGREGATEEXCHANGERATEVOTERESPONSE._serialized_end=507
  _MSGDELEGATEFEEDCONSENT._serialized_start=509
  _MSGDELEGATEFEEDCONSENT._serialized_end=621
  _MSGDELEGATEFEEDCONSENTRESPONSE._serialized_start=623
  _MSGDELEGATEFEEDCONSENTRESPONSE._serialized_end=655
  _MSG._serialized_start=658
  _MSG._serialized_end=1085
# @@protoc_insertion_point(module_scope)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.message
import google.protobuf.timestamp_pb2
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class ReserveSnapshotSavedEvent(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    PAIR_FIELD_NUMBER: builtins.int
    QUOTE_RESERVE_FIELD_NUMBER: builtins.int
    BASE_RESERVE_FIELD_NUMBER: builtins.int
    pair: typing.Text
    quote_reserve: typing.Text
    base_reserve: typing.Text
    def __init__(self,
        *,
        pair: typing.Text = ...,
        quote_reserve: typing.Text = ...,
        base_reserve: typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["base_reserve",b"base_reserve","pair",b"pair","quote_reserve",b"quote_reserve"]) -> None: ...
global___ReserveSnapshotSavedEvent = ReserveSnapshotSavedEvent

class SwapQuoteForBaseEvent(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    PAIR_FIELD_NUMBER: builtins.int
    QUOTE_AMOUNT_FIELD_NUMBER: builtins.int
    BASE_AMOUNT_FIELD_NUMBER: builtins.int
    pair: typing.Text
    quote_amount: typing.Text
    base_amount: typing.Text
    def __init__(self,
        *,
        pair: typing.Text = ...,
        quote_amount: typing.Text = ...,
        base_amount: typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["base_amount",b"base_amount","pair",b"pair","quote_amount",b"quote_amount"]) -> None: ...
global___SwapQuoteForBaseEvent = SwapQuoteForBaseEvent

class SwapBaseForQuoteEvent(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    PAIR_FIELD_NUMBER: builtins.int
    QUOTE_AMOUNT_FIELD_NUMBER: builtins.int
    BASE_AMOUNT_FIELD_NUMBER: builtins.int
    pair: typing.Text
    quote_amount: typing.Text
    base_amount: typing.Text
    def __init__(self,
        *,
        pair: typing.Text = ...,
        quote_amount: typing.Text = ...,
        base_amount: typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["base_amount",b"base_amount","pair",b"pair","quote_amount",b"quote_amount"]) -> None: ...
global___SwapBaseForQuoteEvent = SwapBaseForQuoteEvent

class MarkPriceChanged(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    PAIR_FIELD_NUMBER: builtins.int
    PRICE_FIELD_NUMBER: builtins.int
    TIMESTAMP_FIELD_NUMBER: builtins.int
    pair: typing.Text
    price: typing.Text
    @property
    def timestamp(self) -> google.protobuf.timestamp_pb2.Timestamp: ...
    def __init__(self,
        *,
        pair: typing.Text = ...,
        price: typing.Text = ...,
        timestamp: typing.Optional[google.protobuf.timestamp_pb2.Timestamp] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["timestamp",b"timestamp"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["pair",b"pair","price",b"price","timestamp",b"timestamp"]) -> None: ...
global___MarkPriceChanged = MarkPriceChanged
